// create the module and name it testFlipInsight

var testFlipInsight = angular.module('testFlipInsight',
	['angular-loading-bar',
	'ui.router',
	// 'ngMaterial',
	'toastr',
	'ngAnimate',
	'ngMessages',
	'ngBootbox',
	'angular-jwt',
	'auth0.auth0',
	'RecursionHelper']);

// configure our routes
testFlipInsight.config(['$stateProvider', '$httpProvider', 'jwtOptionsProvider', 'jwtInterceptorProvider', 'angularAuth0Provider', function($stateProvider, $httpProvider, jwtOptionsProvider, jwtInterceptorProvider, angularAuth0Provider, notificationsConfigProvider) {

	// JWT Config
	jwtOptionsProvider.config({
		 whiteListedDomains: ['.+'],

		 authHeader: 'Authorization',

		 authPrefix: 'Bearer ',

		 unauthenticatedRedirectPath: '/login',

		 tokenGetter: function($state, options, jwtHelper, authService, TenantService) {
       if ($state.current.name != 'login') {
				 if (!authService.getIdToken()) {
						 chrome.runtime.sendMessage({operation: "stop"}, function(response) {
							 console.log(response);
						 });

					  $state.go('login');
				 } else if (jwtHelper.isTokenExpired(authService.getIdToken())) {
						 chrome.runtime.sendMessage({operation: "stop"}, function(response) {
			 				console.log(response);
			 			});

           $state.go('login', {errorMessage: 'Authentication expired'});
         }
       }

       return authService.getIdToken();
     }
	 });
	 $httpProvider.interceptors.push('jwtInterceptor');

	// Routes through system
	$stateProvider
		// route for the home page
		.state('projectContext', {
			url: '/context',
			templateUrl : 'app/pages/projectReleaseSelection/contextSelectionView.html',
			controller  : 'contextSelectionController',
			data: {
				requiresLogin: true,
				requiresProjectConfig: false
			}
		})

		// route for the session page
		.state('recordsession', {
			url: '/recordsession',
			templateUrl : 'app/pages/recorder/recordSessionView.html',
			controller  : 'recordSessionController',
			data: {
				requiresLogin: true,
				requiresProjectConfig: true
			}
		})

		.state('reviewScan', {
			url: '/reviewscan',
			templateUrl : 'app/pages/reviewScan/reviewScanView.html',
			controller  : 'reviewScanController',
			data: {
				requiresLogin: true,
				requiresProjectConfig: true
			}
		})

		.state('scanSettings', {
			url: '/scansettings',
			templateUrl : 'app/pages/scanSettings/scanSettings.html',
			controller  : 'scanSettingsController',
			data: {
				requiresLogin: true,
				requiresProjectConfig: false
			}
		})

		.state('reviewSession', {
			url: '/reviewsession',
			templateUrl : 'app/pages/reviewSession/reviewSessionView.html',
			controller  : 'reviewSessionController',
			data: {
				requiresLogin: true,
				requiresProjectConfig: true
			}
		})

		.state('login', {
			url: '/login?errorMessage',
			templateUrl : 'app/pages/login/loginView.html',
			controller  : 'loginController',
			data: {
				requiresLogin: false,
				requiresProjectConfig: false
			}
		})
}]);

testFlipInsight.run(['$rootScope','$transitions', '$state','authService', 'TenantService', 'API_URL', function run($rootScope, $transitions, $state, authService, TenantService, API_URL) {
  // Handle the authentication
  // result in the hash
  authService.handleAuthentication();
	// authService.scheduleRenewal();

	// LOAD previous settings to $rootScope
	$rootScope.serviceDetails = {};
	chrome.storage.local.get({service_settings : {}}, (state) => {
		if (state.service_settings != null) {
			// Assign
			$rootScope.serviceDetails = state.service_settings;

			if ($rootScope.serviceDetails.workspace != null) {
				TenantService.company = $rootScope.serviceDetails.workspace;
			}

			if ($rootScope.serviceDetails.API_URL != null) {
				API_URL.URL = $rootScope.serviceDetails.API_URL;
			}
		}

		// Route to appropriate page
		if (!authService.isAuthenticated()) {
				$state.go('login');
		} else {
				// If !details OR workspace defined -> send to context page to set it
				if (($rootScope.serviceDetails.project_id == null) || ($rootScope.serviceDetails.release_id == null) || ($rootScope.serviceDetails.workspace == null)) {
					$state.go('projectContext');
				} else {
					// else -> send to recording page
					$state.go('recordsession');
				}
		}
	});


	// Event for transitions
	$transitions.onStart({} , function (transition) {
	    if (transition.$to().data && transition.$to().data.requiresLogin && (!authService.isAuthenticated())) {
	      $state.go('login');
	    }

			if (transition.$to().data && transition.$to().data.requiresProjectConfig) {
				if (($rootScope.serviceDetails.project_id == null) || ($rootScope.serviceDetails.release_id == null) || ($rootScope.serviceDetails.workspace == null)) {
					$state.go('projectContext');
			  }
		  }
		});
  }
]);

testFlipInsight.controller('mainController', ['$scope', '$rootScope','$transitions', '$state',
		function($scope, $rootScope, $transitions, $state) {
			$rootScope.serviceDetails = {};
			$rootScope.objects_list = [];

			var manifestData = chrome.runtime.getManifest();

			$scope.version = manifestData.version;
		}
]);
