
testFlipInsight.controller('contextSelectionController', ['$rootScope','$scope', '$state', '$http', 'TestAdminService', 'TestProjectService', 'TestReleaseService', 'TenantService', 'authService', 'API_URL', 'APP_CONFIG', 'toastr',
	function($rootScope, $scope, $state, $http, TestAdminService, TestProjectService, TestReleaseService, TenantService, authService, API_URL, APP_CONFIG, toastr) {
		/* On load */
		$scope.workspace_options = [];
		$scope.project_options = [];
		$scope.release_options = [];

		$scope.recorderEnabled = APP_CONFIG.recorderEnabled;

		// Set defaults to form
		loadWorkspaces();

		function loadWorkspaces()
		{
			TestAdminService.getAllTenantsForUser(authService.getUserId())
			.then(function (data, status, headers, config) {

				$scope.workspace_options = data.data;

				if ($scope.workspace_options.length > 0) {
					$scope.selected_workspace = $scope.workspace_options[0];

					loadWorkspace($scope.workspace_options[0].tenantEntity.tenantId);
				}
			},function (fdata, status, header, config) {
				TestAdminService.getTenantsForUser(authService.getUserId())
				.then(function (data, status, headers, config) {

					$scope.workspace_options = data.data;

					if ($scope.workspace_options.length > 0) {
						$scope.selected_workspace = $scope.workspace_options[0];

						loadWorkspace($scope.workspace_options[0].tenantEntity.tenantId);
					}
				},function (data, status, header, config) {

					console.log(data);
				});
			});
		}

		$scope.onWorkspaceSelected = function()
		{
			if ($scope.selected_workspace == null)
				return;

			loadWorkspace($scope.selected_workspace.tenantEntity.tenantId);
		}

		function loadWorkspace(id)
		{
			TenantService.company = id;

			loadProjects();
		}

		function loadProjects () {
			TestProjectService.getAllProjects()
			.then(function (data, status, headers, config) {
				$scope.project_options = data.data;
				if ($scope.project_options.length > 0) {
					$scope.selected_project = $scope.project_options[0];

					loadRelease($scope.selected_project.id);
				}
			},function (data, status, header, config) {
				$scope.project_options = {};

				toastr.error("We are unable to load the projects available. Please try again.", "Unable to load projects");
			});
		}

		$scope.onProjectSelected = function () {
			loadRelease($scope.selected_project.id);
		}

		function loadRelease(project_id) {
			TestReleaseService.getAllReleasesByProjectID(project_id)
			.then(function (data, status, headers, config) {
				$scope.release_options = data.data;
				if ($scope.release_options.length > 0) {
					$scope.selected_release = $scope.release_options[0]
				}

			},function (data, status, header, config) {
				$scope.release_options = {};

				toastr.error("We are unable to load the releases available for the specified project. Please try again.", "Unable to load releases");
			});
		}

		/* On form submit */
		$scope.submitForm = function (ext_mode) {
			if (ext_mode === "settings") {

				$state.go('scanSettings');

			} else {
				if ((!$scope.selected_project) || (!$scope.selected_release) || (!$scope.selected_workspace))
					return;

				// extract session settings
				$rootScope.serviceDetails.workspace = $scope.selected_workspace.tenantEntity.tenantId;
				$rootScope.serviceDetails.project_id = $scope.selected_project.id;
				$rootScope.serviceDetails.release_id = $scope.selected_release.id;
				$rootScope.serviceDetails.API_URL = API_URL.URL;

				$scope.serviceDetails = $rootScope.serviceDetails;


				// Store in chrome storage
				chrome.storage.local.set({ ext_mode: ext_mode, service_settings: $scope.serviceDetails});

				// forward to recordsession tab
				$state.go('recordsession');
			}
		}

		/* Form reset */
		$scope.resetForm = function () {
			$scope.workspace_options = {};
			$scope.release_options = {};
			$scope.project_options = {};

			loadWorkspaces();
		}
}]);
