testFlipInsight.controller('recordSessionController', ['$rootScope','$scope', '$compile', '$state', '$http', 'JourneyService', 'TestSessionService', 'TestSessionActivityService', 'TestSessionConsoleService', 'TestSessionRRPairService', 'ExpectedResultService', 'folderService', 'fileService', 'pageCollectionService', 'journeyPoolService', 'automationFrameworkService', 'APP_CONFIG', '$ngBootbox', 'toastr',
	function($rootScope, $scope, $compile, $state, $http, JourneyService, TestSessionService, TestSessionActivityService, TestSessionConsoleService, TestSessionRRPairService, ExpectedResultService, folderService, fileService, pageCollectionService, journeyPoolService, automationFrameworkService, APP_CONFIG, ngBootbox, toastr)
	{
		// Set defaults
		$scope.stopwatchControl = null;
		$scope.recorded_items = 0;
		$scope.record_status = 'Stopped';

		$scope.recorderEnabled = APP_CONFIG.recorderEnabled;

		// Load record settings
		chrome.storage.local.get({
			operation: 'stop',
			start_time: new Date(),
			end_time: new Date(),
			ext_mode: 'scanner',
			action: [],
			console: [],
			rr_pairs: [],
			scanned_items: [],
			start_url: null
		}, (state) => {
			// set status
			if (state.operation === 'record') {
				$scope.$apply( function() {
					$scope.record_status = 'Recording';
				});
			} else if (state.operation === 'record_scanner') {
				$scope.$apply( function() {
					$scope.record_status = 'Scanning';
				});
			} else if (state.operation === 'pause') {
				$scope.$apply( function() {
					$scope.record_status = 'Paused';
				});
			} else {
				$scope.$apply( function() {
					$scope.record_status = 'Stopped';
				});
			}

			if (state.ext_mode != null) {
				$scope.extMode = state.ext_mode;
			}

			// retrieve time
			if (state.start_time != null) {
				if (state.operation === 'record' || state.operation === 'record_scanner') {
					var cur_date = new Date(state.start_time);

					if ($scope.stopwatchControl != null) {
						$scope.stopwatchControl.setStartTime(cur_date);
						$scope.stopwatchControl.start();
					}
				} else if (state.end_time != null && (state.action.length > 0 || state.scanned_items.length > 0)) {
					var cur_date = new Date(state.start_time);
					var end_date = new Date(state.end_time);

					if ($scope.stopwatchControl != null) {
						$scope.stopwatchControl.setStartTime(cur_date);
						$scope.stopwatchControl.setEndTime(end_date);
					}
				}
			}

			// set objects
			$rootScope.testActivity = state.action;
			$scope.$apply(function() { $scope.recorded_items = $rootScope.testActivity.length; });

			$rootScope.scanned_items = state.scanned_items;
			$rootScope.url = state.start_url;

			$scope.$apply(function() { $scope.no_scanned_items = $rootScope.scanned_items.length; });

			$rootScope.recorderConsole = state.console;
			$rootScope.recorderRRPairs = state.rr_pairs;
		});

		$scope.modeAdjusted= function()
		{
			chrome.storage.local.set({ext_mode: $scope.extMode});
		}

		/* Scanner */
		$scope.start_scanning = function ()
		{
			// Start recording
			if ($scope.stopwatchControl != null) {
				$scope.stopwatchControl.clear();

				$scope.stopwatchControl.start();

				// Store start time
				chrome.storage.local.set({start_time: $scope.stopwatchControl.startTime().toString()});
			}

			$scope.record_status = 'Scanning';

			chrome.tabs.query({'active': true, 'lastFocusedWindow': true}, function (tabs) {
				var url = tabs[0].url;

				// Send message to background
				chrome.runtime.sendMessage({operation: "record_scanner", start_url: url}, function(response) {
					console.log(response);
				});

				// Close window
				window.close();
			});

		}

		$scope.stop_scanning = function()
		{
			if ($scope.stopwatchControl != null) {
				$scope.stopwatchControl.stop();

				chrome.storage.local.set({end_time: $scope.stopwatchControl.endTime().toString()});
			}

			$scope.record_status = 'Stopped';

			chrome.runtime.sendMessage({operation: "stop"}, function(response) {
				console.log(response);
			});
		}

		$scope.upload_scan = function()
		{
			if ($scope.containsErrors()) {
				ngBootbox.confirm({message: "We recommend you review all scanned objects and warnings.",
					title: 'Scan Contains Warnings',
					closeButton: false,
					buttons: {
				         confirm: {
				             label: 'Review Scan',
				             className: 'btn-success'
				         },
				         cancel: {
				             label: 'Upload Anyway',
				             className: 'btn-danger'
				         }
				     }
				})
		    .then(function() {
		        $scope.review_scan();
		    }, function() {
		        uploadScan();
		    });
			} else {
				uploadScan();
			}
		}


		function uploadScan()
		{
			chrome.tabs.query({'active': true, 'lastFocusedWindow': true}, function (tabs) {
				var pObjects = [];
				for (var i = 0; i < $rootScope.scanned_items.length; i++) {
					var curObj = JSON.parse(JSON.stringify($rootScope.scanned_items[i].currentObject.obj));

					if ($rootScope.scanned_items[i].currentObject.obj.objectType !== "Link" && $rootScope.scanned_items[i].currentObject.obj.objectType !== "Button" && $rootScope.scanned_items[i].currentObject.obj.objectType !== "Select" && $rootScope.scanned_items[i].currentObject.obj.objectType !== "Input" && $rootScope.scanned_items[i].currentObject.obj.objectType !== "ExistsAssertion") {
						curObj.customObjectType = $rootScope.scanned_items[i].currentObject.obj.objectType;

						curObj.objectType = "Custom";
					} else {
						curObj.customObjectType = null;
					}

					pObjects.push(curObj);
				}


				var page_object = {};
				page_object.name = tabs[0].title;

				if ($rootScope.url != null)
					page_object.url = $rootScope.url;
				else
					page_object.url = tabs[0].url;

				page_object.pageObjects = pObjects;

				$scope.confirmPageCollectionDir.open(page_object, $rootScope.serviceDetails.release_id);
			});
		}

		$scope.review_scan = function()
		{
			$state.go('reviewScan');
		}

		$scope.reset_scan = function()
		{
			$scope.record_status = 'Stopped';

			chrome.runtime.sendMessage({operation: "stop"}, function(response) {
				$rootScope.scanned_items = [];

				if ($scope.stopwatchControl != null) {
					$scope.stopwatchControl.stop();
					$scope.stopwatchControl.clear();
				}

				$scope.no_scanned_items = 0;

				chrome.storage.local.set({
					operation: 'stop',
					scanned_items: [],
					start_time: null,
					end_time: null});
			});

		}

		/* Record session */
		$scope.start_recording = function () {
			if (!$scope.recorderEnabled)
				return;

			// Start recording
			if ($scope.stopwatchControl != null) {
				$scope.stopwatchControl.clear();

				$scope.stopwatchControl.start();

				// Store start time
				chrome.storage.local.set({start_time: $scope.stopwatchControl.startTime().toString()});
			}

			$scope.record_status = 'Recording';


			// Close window
			window.close();

			// Send message to background
			chrome.runtime.sendMessage({operation: "record"}, function(response) {
				console.log(response);
			});
		};

		$scope.resume_scan = function () {
			if ($scope.stopwatchControl != null) {
				$scope.stopwatchControl.start();
			}

			$scope.record_status = 'Scanning';

			// Send message to background
			chrome.runtime.sendMessage({operation: "resume_scan"}, function(response) {
				console.log(response);
			});
		}

		$scope.containsErrors = function() {
			if ($rootScope.scanned_items != null) {
				for (var i = 0; i < $rootScope.scanned_items.length; i++) {
					var allWarning = true;
					for (var j = 0; j < $rootScope.scanned_items[i].currentObject.obj.parameters.length; j++) {
						allWarning = allWarning && $rootScope.scanned_items[i].currentObject.obj.parameters[j].identiferWarning;
					}

					if (allWarning || $rootScope.scanned_items[i].currentObject.obj.nameWarning || $rootScope.scanned_items[i].currentObject.obj.iframeWarning) {
						return true;
					}
				}
			}

			return false;
		}

		$scope.resume_recording = function () {
			if ($scope.stopwatchControl != null) {
				$scope.stopwatchControl.start();
			}

			$scope.record_status = 'Recording';

			// Send message to background
			chrome.runtime.sendMessage({operation: "resume"}, function(response) {
				console.log(response);
			});
		}

		$scope.pause_recording = function() {
			if ($scope.stopwatchControl != null) {
				$scope.stopwatchControl.stop();
			}

			$scope.record_status = 'Paused';

			chrome.runtime.sendMessage({operation: "pause"}, function(response) {
				console.log(response);
			});
		}

		/* Stop session */
		$scope.stop_recording = function () {
			if (!$scope.recorderEnabled)
				return;

			if ($scope.stopwatchControl != null) {
				$scope.stopwatchControl.stop();

				chrome.storage.local.set({end_time: $scope.stopwatchControl.endTime().toString()});
			}

			$scope.record_status = 'Stopped';


			chrome.runtime.sendMessage({operation: "stop"}, function(response) {
				console.log(response);
			});
		};

		/* Review session */
		$scope.review_session = function () {
			if (!$scope.recorderEnabled)
				return;

			$state.go('reviewSession');
		};

		/* Scan Page */
		$scope.scan_page = function() {
			chrome.runtime.sendMessage({operation: "scan_page"}, function(response) {
					var confirm = {
								controller: confirmPageCollectionController,
					      templateUrl: 'app/pages/recorder/dialogs/confirmPageCollection.html',
								locals : {
									compile: $compile,
									pageObject: response.data,
									folderService: folderService,
									fileService: fileService,
									pageCollectionService: pageCollectionService,
									releaseId: $rootScope.serviceDetails.release_id
								},
					      parent: angular.element(document.body)
					   };

					   $mdDialog.show(confirm).then(function() {
					   });

				 return true;
			});
		}

		/* Reset session */
		$scope.reset_session = function () {
			if (!$scope.recorderEnabled)
				return;

				$scope.record_status = 'Stopped';

				chrome.runtime.sendMessage({operation: "clear"}, function(response) {
					console.log(response);
				});

				$rootScope.testActivity = [];
				$rootScope.recorderConsole = [];
				$rootScope.recorderRRPairs = {};

				if ($scope.stopwatchControl != null) {
					$scope.stopwatchControl.stop();
					$scope.stopwatchControl.clear();
				}

				$scope.recorded_items = 0;

				chrome.storage.local.set({
							operation: 'stop',
				 			action: [],
							console: [],
							rr_pairs: [],
							start_time: null,
							end_time: null});
		}

		$scope.uploadSession = function () {
			if (!$scope.recorderEnabled)
				return;

			if ($rootScope.testActivity.length == 0) {
				toastr.error("There is no test activity recorded. Please record a test session then upload.", "No Test Activity.");

  			return;
			}

			$scope.confirmSessionCollectionDir.open($rootScope.serviceDetails.project_id,
																							$rootScope.serviceDetails.release_id,
																							$rootScope.testActivity,
																							$rootScope.recorderRRPairs,
																							$rootScope.recorderConsole);
		}

		// ******** Called on action list changing *********
		chrome.storage.onChanged.addListener((changes, _) =>
		{
		  for (const key in changes) {
			if (key === 'action') {
				$rootScope.testActivity = changes.action.newValue;
				$scope.$apply(function() { $scope.recorded_items = $rootScope.testActivity.length; });
			} else if (key === 'rr_pairs') {
				$rootScope.recorderRRPairs = changes.rr_pairs.newValue;
			} else if (key === 'console') {
				$rootScope.recorderConsole = changes.console.newValue;
			}
		}
		});
	}
]);
