testFlipInsight.service('authService', ['$state', '$http', 'jwtHelper', 'angularAuth0', '$timeout', 'AUTH_CONFIG', 'API_URL',
  function authService($state, $http, jwtHelper, angularAuth0, $timeout, AUTH_CONFIG, API_URL) {

    var tokenRenewalTimeout

    function setAccessToken(token)
    {
      localStorage.setItem('access_token', token);
    }

    function setIdToken(token)
    {
      localStorage.setItem('id_token', token);
    }

    function renewToken(callback) {
      angularAuth0.checkSession({ },
        function(err, result) {
          if (!err) {
            setSession(result);
          }

          if (callback != null) {
              callback(err, result)
          }
        }
      );
    }

    function login(api, username, password, callback)
    {
      logout();

      if (AUTH_CONFIG.nativeAuth) {
        var config = {
          headers : {
            'Content-Type': 'application/json'
          }
        }

        $http.post(api + "/api/administration/authorization/login", {email: username, password: password},config)
        .then (function(data, status, headers, config) {
          callback(false, data.data);
        }, function(data, status, headers, config) {
          callback(true, data.data);
        });
      } else {
        angularAuth0.client.login({
          realm: 'Username-Password-Authentication',
          username: username,
          password: password,
        }, function(err, authResult) {
          callback(err, authResult);
        });
      }
    }

    function handleAuthentication() {
      angularAuth0.parseHash(function(err, authResult) {
        if (authResult && authResult.idToken) {
          setSession(authResult);
          $state.go('home');
        } else if (err) {
          $timeout(function() {
            $state.go('home');
          });

          $state.go('login');
        }
      });
    }

    function setSession(authResult) {
      // Set the time that the access token will expire at
      let expiresAt = JSON.stringify((authResult.expiresIn * 1000) + new Date().getTime());
      localStorage.setItem('access_token', authResult.accessToken);
      localStorage.setItem('id_token', authResult.idToken);
      localStorage.setItem('refresh_token', authResult.refreshToken);
      localStorage.setItem('expires_at', expiresAt);
    }

    function logout() {
      // Remove tokens and expiry time from localStorage
      localStorage.removeItem('access_token');
      localStorage.removeItem('id_token');
      localStorage.removeItem('expires_at');
      localStorage.removeItem('refresh_token');

      clearTimeout(tokenRenewalTimeout);
    }

    function isAuthenticated() {
      if (getIdToken() == null)
        return false;

      if (jwtHelper.isTokenExpired(getIdToken())) {
        return false;
      }

      // Check whether the current time is past the
      // access token's expiry time
      let expiresAt = JSON.parse(localStorage.getItem('expires_at'));
      return new Date().getTime() < expiresAt;
    }

    function isAuthorized(tenantName) {
      // Get access tokens
      var token = localStorage.getItem('id_token');

      if (!token)
        return false;

      // Get current sub-domain

      return userHasAccessToTenant(tenantName, token);
    }

    function getUserId() {
        var tokenID = localStorage.getItem('id_token');

        if (!tokenID)
          return null;

        var tokenPayload = jwtHelper.decodeToken(tokenID);
        if (!tokenPayload['sub'])
          return null;

        return tokenPayload['sub'];
    }

    function getIdToken() {
      var tokenID = localStorage.getItem('id_token');

      return tokenID;
    }

    function userHasAccessToTenant(tenantId, tokenID)
    {
      var tokenPayload = jwtHelper.decodeToken(tokenID);

      if (!tokenPayload['http://testflip.co.uk/auth'])
        return false;

      var tokenList = tokenPayload['http://testflip.co.uk/auth'];
      for (var i = 0; i < tokenList.length; i++) {
        if (tokenList[i]['tenantEntity']['tenantId']) {
          if (tenantId == tokenList[i]['tenantEntity']['tenantId']) {
            return true;
          }
        }
      }

      return false;
    }

    function getTenants()
    {
      var tokenID = localStorage.getItem('id_token');

      return getTenantsByToken(tokenID);
    }

    function getTenantsByToken(tokenID)
    {
      if (!tokenID)
        return [];

      var tokenPayload = jwtHelper.decodeToken(tokenID);

      if (!tokenPayload['http://testflip.co.uk/auth'])
        return [];

      var tokenList = tokenPayload['http://testflip.co.uk/auth'];
      var tenantList = [];

      for (var i = 0; i < tokenList.length; i++) {
        if (tokenList[i]['tenantEntity']) {
          tenantList.push(tokenList[i]['tenantEntity']);
        }
      }

      return tenantList;
    }

    return {
      setAccessToken: setAccessToken,
      setIdToken: setIdToken,
      getIdToken: getIdToken,
      login: login,
      getUserId: getUserId,
      isAuthorized: isAuthorized,
      handleAuthentication: handleAuthentication,
      logout: logout,
      renewToken: renewToken,
      userHasAccessToTenant: userHasAccessToTenant,
      getTenants: getTenants,
      getTenantsByToken: getTenantsByToken,
      setSession: setSession,
      isAuthenticated: isAuthenticated
    };
}]);
