testFlipInsight.directive('stopwatch', function($timeout) {
    return {
      restrict: 'E',

	  transclude: true,

	  scope: {
	    control: '='
	  },

	  link: function(scope, element, attrs) {
		scope.internalControl = scope.control || {};

        var timeoutId;
        scope.seconds = 0;
        scope.minutes = 0;
		scope.hours = 0;
		scope.startTime = new Date();
		scope.endTime = null;
        scope.running = false;

        scope.internalControl.stop = function() {
          $timeout.cancel(timeoutId);
          scope.running = false;
		  scope.endTime = new Date();
        };

      scope.internalControl.start = function() {
		       timer();

			     scope.running = true;
      };

      scope.internalControl.clear = function() {
          scope.seconds = 0;
          scope.minutes = 0;
    		  scope.hours = 0;
    		  scope.endTime = null;
    		  scope.startTime = new Date();
        };

		scope.internalControl.setStartTime = function (start) {
			scope.startTime = start;

			updateTime();
		}

		scope.internalControl.setEndTime = function (end) {
			scope.endTime = end;

			updateTime();
		}

		scope.internalControl.startTime = function() {
			return scope.startTime;
		};

		scope.internalControl.endTime = function() {
			return scope.endTime;
		};

        function timer() {
          timeoutId = $timeout(function() {
            updateTime(); // update Model
            timer();
          }, 1000);
        }

        function updateTime() {
			var endTime;
			if (scope.endTime != null) {
				endTime = scope.endTime;
			} else {
				endTime = new Date();
			}

			var timeDiff = endTime - scope.startTime; //in ms

			var milliseconds = parseInt((timeDiff%1000)/100)
            , seconds = parseInt((timeDiff/1000)%60)
            , minutes = parseInt((timeDiff/(1000*60))%60)
            , hours = parseInt((timeDiff/(1000*60*60))%24);

			scope.$apply( function() {

			scope.hours = (hours < 10) ? "0" + hours : hours;
			scope.minutes = (minutes < 10) ? "0" + minutes : minutes;
			scope.seconds = (seconds < 10) ? "0" + seconds : seconds;
			});
        }
      },
      template:
        '<div class="blueborder">' +
          '<div>{{hours|numberpad:2}}:{{minutes|numberpad:2}}:{{seconds|numberpad:2}}</div><br/>' +
        '</div>',
      replace: true
    };
  }).
  filter('numberpad', function() {
    return function(input, places) {
      var out = "";
      if (places) {
        var placesLength = parseInt(places, 10);
        var inputLength = input.toString().length;

        for (var i = 0; i < (placesLength - inputLength); i++) {
          out = '0' + out;
        }
        out = out + input;
      }
      return out;
    };
  });
