/* Background javascript code which is executed in extension */
/* Background.js maintains a list of actions recorded */


let recorder_actions = [];
let recorder_console = [];
let recorder_rr = [];
let scanned_items = [];
let unique_scanned_items = [];

var operation = 'stop';

let extState = null;

chrome.storage.local.set({ operation: 'stop', action: [], console: [], recorder_actions: [], rr_pairs: {}});

// Get current time
function getTime() {
  return new Date().getTime();
}


function addNewError(consoleMsg) {
  chrome.storage.local.get({ operation: 'stop' , action: [] }, (state) => {
    if (state.operation == 'record') {
        if (recorder_console.length > 1) {
          var lastError = recorder_console[recorder_console.length - 1];
          var isSameAsLast = (lastError != undefined) && lastError.message.trim() === consoleMsg.message.trim() && lastError.url.trim() === consoleMsg.url.trim();// && lastError.lineNo == consoleMsg.lineNo && lastError.columnNo == consoleMsg.columnNo && lastError.error == consoleMsg.error;

          if (isSameAsLast)
            return;
        }

      recorder_console.push(consoleMsg);

      chrome.storage.local.set({ console: recorder_console}, function(data) {
        if(chrome.runtime.lastError){
          alert(chrome.runtime.lastError.message);
        }
      });
    }
  });
}

function addNewRRPair(rrID, rrPair) {
  chrome.storage.local.get({ operation: 'stop' , action: [] }, (state) => {
    if (state.operation == 'record') {
      recorder_rr.push(JSON.stringify(rrPair));

      saveRRPairs();
    }
  });
}

function saveRRPairs()
{
  chrome.storage.local.set({ rr_pairs: recorder_rr}, function(data) {
    if(chrome.runtime.lastError){
      saveRRPairs();
    }
  });
}

function addNewAction(action) {
  if (action.type === undefined)
    action.type = "InteractionAction";

  addAction(action);
}

function addEventAction(action) {
  if (recorder_actions.length >= 1) {
    if (recorder_actions[recorder_actions.length - 1].title == undefined || recorder_actions[recorder_actions.length - 1].title.trim() === "") {
      recorder_actions[recorder_actions.length - 1].title = action.title;
    }
  }

  addAction(action);
}

function addAction(action)
{
  var save_action = action;

  chrome.tabs.captureVisibleTab(null, {format: "jpeg", quality: 5}, function(data) {
    if (save_action.action_img != null && save_action.action_img != undefined) {
      save_action.action_img.screenshot = data;
    }

    recorder_actions.push(save_action);

    chrome.storage.local.set({action: recorder_actions}, function(data) {
      if(chrome.runtime.lastError){
        alert(chrome.runtime.lastError.message);
      }
    });
  });

  // chrome.notifications.create('', {
  //   type: 'basic',
  //   iconUrl: '/assets/sel_mini_logo.png',
  //   title: 'Action Registered',
  //   message: ('Type: ' + action.code.type + '\nElement: ' + action.code.elem + '\nValue: ' + action.code.value)
  // }, function(notificationId) {});
}

function addNewUrlActionByURL(url,title, open)
{
	var code;

  if (open) {
    code = CodeObject.ConstructObject(Code_Actions.Open_URL, url);
  } else {
    code = CodeObject.ConstructObject(Code_Actions.END_URL, url);
  }

  chrome.tabs.getSelected(null, function(tab) {
      chrome.tabs.sendRequest(tab.id, {method: "getScreenData"}, function(response) {
          var img = null;

          if(response.method=="getScreenData"){
            var img = response.data;

          	var eng_text;

            if (open) {
              eng_text =  "Open URL '" + url + "'";
            } else {
              eng_text =  "Current URL '" + url + "'";
            }
          }

          var action_event = ActionObject.ConstructObject(eng_text, eng_text, "", url, title, getTime(), code, img, null, null, null, null, null);
          action_event.type = "EventAction";

          addNewAction(action_event);
      });
  });

}


function addNewUrlAction(tab)
{
	addNewUrlActionByURL(tab.url, tab.title, true);
}

// Message listener from main_page.js and content_script.js
chrome.runtime.onMessage.addListener(
  (request, sender, sendResponse) => {
    operation = request.operation;
if (operation === 'record') {
      extState = 'record';

  	   // Change icon
  	   chrome.browserAction.setIcon({path: "/assets/record_icon.png"});

  		// Parse message to each tab
  		chrome.tabs.query({active: true, currentWindow: true}, function(tabs) {

  			// Define empty list
  			recorder_actions = [];

  			// Add open URL action
  			addNewUrlAction(tabs[0]);

  			// Send message to current tab open to record
  			chrome.tabs.sendMessage(tabs[0].id, {  operation: operation, record_option:  request.record_option}, function(response) {
  				console.log(response);
  			});

        addRecordingListeners();

  			// Set storage to record
  			chrome.storage.local.set({ operation: 'record', action: recorder_actions, console: recorder_console, rr_pairs: recorder_rr}, function(data) {
  				if(chrome.runtime.lastError){
  					alert(chrome.runtime.lastError.message);
  				}
  			});
		});
  } else if (operation === 'record_scanner') {
    extState = 'record_scanner';

    // Change icon
      chrome.browserAction.setIcon({path: "/assets/record_icon.png"});
     // Parse message to each tab
     chrome.tabs.query({}, function(tabs) {
       // Define empty list
       scanned_items = [];
       unique_scanned_items = [];

       chrome.storage.local.get({ selectorType: null, customScan: null, searchDepth: 4, objectIndex: false }, (state) => {
         for (var i=0; i<tabs.length; ++i) {
           chrome.tabs.sendMessage(tabs[i].id, {  operation: operation, objectIndex: state.objectIndex, searchDepth: state.searchDepth, selector_type: state.selectorType, custom_scan: state.customScan}, function(response) {
             console.log(response);
           });

           chrome.storage.local.set({ operation: 'record_scanner', scanned_items: scanned_items}, function(data) {
             if(chrome.runtime.lastError) {
               alert(chrome.runtime.lastError.message);
             }
           });
         }
       });

     });
  } else if (operation === 'scan_page') {
    chrome.tabs.query({active: true, currentWindow: true}, function(tabs) {
      chrome.tabs.sendRequest(tabs[0].id, {  operation: 'getScreenScrape' }, function(responsed) {
        if(responsed.method === "returngetScreenScrape"){

          sendResponse({data: responsed.screendata, method: "scan_page"} );
        }
      });
    });

    return true;
  } else if (operation === 'resume') {
        extState = 'record';

        // Change icon
        chrome.browserAction.setIcon({path: "/assets/record_icon.png"});

       // Parse message to each tab
       chrome.tabs.query({active: true, currentWindow: true}, function(tabs) {
         // Send message to current tab open to record
         chrome.tabs.sendMessage(tabs[0].id, {  operation: operation }, function(response) {
           console.log(response);
         });

         addRecordingListeners();

         // Set storage to record
         chrome.storage.local.set({ operation: 'record', action: recorder_actions, console: recorder_console, rr_pairs: recorder_rr}, function(data) {
           if(chrome.runtime.lastError){
             alert(chrome.runtime.lastError.message);
           }
         });
     });
  } else if (operation === 'resume_scan') {
    if (extState !== 'record_scanner')
      return true;

    extState = 'record_scanner';


    chrome.browserAction.setIcon({path: "/assets/record_icon.png"});

    // Parse message to each tab
    chrome.tabs.query({}, function(tabs) {
      for (var i=0; i<tabs.length; ++i) {
        chrome.tabs.sendMessage(tabs[i].id, {  operation: 'record_scanner', custom_scan: request.custom_scan}, function(response) {
          console.log(response);
        });

        chrome.storage.local.get({
          scanned_items: []
        }, (state) => {
          scanned_items = state.scanned_items;

          chrome.storage.local.set({ operation: 'record_scanner', custom_scan: request.custom_scan, scanned_items: scanned_items}, function(data) {
            if(chrome.runtime.lastError) {
              alert(chrome.runtime.lastError.message);
            }
          });
        });
      }
    });
  } else if (operation === 'pause') {
    if (extState !== 'record_scanner' && extState !== 'record') {
      return true;
    }

    chrome.browserAction.setIcon({path: "/assets/icon.png"});

    // Parse message to each tab
    chrome.tabs.query({}, function(tabs) {
      for (var i=0; i<tabs.length; ++i) {
        // Send message to current tab open to record
        chrome.tabs.sendMessage(tabs[i].id, {  operation: 'stop' }, function(response) {
          console.log(response);
        });
      }

      removeRecordingListeners();

      // Set storage to record
      chrome.storage.local.set({ operation: 'pause', action: recorder_actions, console: recorder_console, rr_pairs: recorder_rr, scanned_items: scanned_items}, function(data) {
        if(chrome.runtime.lastError){
          alert(chrome.runtime.lastError.message);
        }
      });
  });

	} else if (operation === 'load') {
		chrome.storage.local.get({ operation: 'stop' , action: [], identifierMaps: [], identifierPreferences:[], identifierAttrs: [], objectIndex: false, searchDepth: 4, selectorType: null, customScan: null }, (state) => {
      if (operation === 'record_scanner' || operation === 'record')
        extState = operation;

			if (sender.tab) {
				chrome.tabs.sendMessage(sender.tab.id, { operation: state.operation, identifier_mappings: state.identifierMaps, objectIndex: state.objectIndex, searchDepth: state.searchDepth,  identifier_preferences: state.identifierPreferences, identifier_attributes: state.identifierAttrs, selector_type: state.selectorType, custom_scan: state.customScan}, function(response) {
					console.log(response);
				});
			}
		});

	} else if (operation === 'stop' || operation === 'clear') {
      extState = null;

      // change icon
  		chrome.browserAction.setIcon({path: "/assets/icon.png"});

  		// Stop in all tabs
      removeRecordingListeners();

      // Parse message to each tab
      if (operation !== 'clear') {
        chrome.tabs.query({active: true, currentWindow: true}, function(tabs) {
          addNewUrlActionByURL(tabs[0].url, tabs[0].title, false);
        });
      }


  		chrome.tabs.query({}, function(tabs) {
  			// Send message to all tabs open to stop
  			for (var i = 0; i < tabs.length; i++) {
  				chrome.tabs.sendMessage(tabs[i].id, { operation: operation }, function(response) {
  					console.log(response);
  				});
  			}
  		});

  		if (operation === 'clear') {
  			recorder_actions = [];
        scanned_items = [];
        unique_scanned_items = [];
  		}

  		// Set storage to stopped
  		chrome.storage.local.set({ operation: 'stop', action:  recorder_actions, scanned_items: scanned_items, console: recorder_console, rr_pairs: recorder_rr}, function(data) {
  			if(chrome.runtime.lastError){
  				alert(chrome.runtime.lastError.message);
  			}
  		});

	// } else if (operation === 'export_bpmn') {
		// ModelBuilder.exportModel(recorder_actions, Model_Builders.BPMN);

	} else if (operation === 'upload_session_async') {
    var uploadState = "PendingAdd";
    var errorReason = "";

    if (request.uploadState != undefined) {
      uploadState = request.uploadState;

      if (request.uploadDesc != undefined)
  			errorReason = request.uploadDesc;
    }

    ActivityUploader.createNewSessionApiKeyAsync(recorder_actions, recorder_rr, recorder_console, request.name, request.source, request.sourceLocation, request.releaseID, request.poolId, request.apiKey, uploadState, errorReason, request.url);

  } else if (operation === 'upload_session_sync') {
    var uploadState = "PendingAdd";
    var errorReason = "";

    if (request.uploadState != undefined) {
      uploadState = request.uploadState;

      if (request.uploadDesc != undefined)
  			errorReason = request.uploadDesc;
    }

    ActivityUploader.createNewSessionApiKeySync(recorder_actions, recorder_rr, recorder_console, request.name, request.source, request.sourceLocation, request.releaseID, request.poolId, request.apiKey, uploadState, errorReason, request.url);
  } else if (operation === 'scraped_object') {
      var save_object = request.object.currentObject.obj;

      if (save_object != null) {
        if (unique_scanned_items.indexOf(save_object.shadowRoot + save_object.iframeXPath + save_object.uniqueXPath) < 0) {
          scanned_items.push(request.object);
          unique_scanned_items.push(save_object.shadowRoot + save_object.iframeXPath + save_object.uniqueXPath);

          chrome.storage.local.set({scanned_items: scanned_items}, function(data) {
            if(chrome.runtime.lastError){
              alert(chrome.runtime.lastError.message);
            }
          });
        } else {
          chrome.notifications.create('', {
            type: 'basic',
            iconUrl: '/assets/sel_mini_logo.png',
            title: 'Object Previously Scanned',
            message: ('Name: ' + save_object.name)
          }, function(notificationId) {});

        }

        // chrome.notifications.create('', {
        //   type: 'basic',
        //   iconUrl: '/assets/sel_mini_logo.png',
        //   title: 'Object Scanned',
        //   message: ('Name: ' + save_object.name + '\n Type: ' + save_object.objectType)
        // }, function(notificationId) {});
      }

  } else if (operation === 'action') {
    addNewAction(request.action);
	} else if (operation === 'console') {
    addNewError(request.error);
  } else if (operation === 'rrpair') {
    addNewRRPair(request.rrpair);
  }

  }

);

function getOperation() {
	return operation;
}

// Pick up new url entered
var add_url_event = false;
if (chrome.webNavigation != null) {
  chrome.webNavigation.onCommitted.addListener(function(details) {
  	if (details.transitionQualifiers.indexOf("from_address_bar") > -1) {

  		chrome.storage.local.get({ operation: 'stop' , action: [] }, (state) => {
  			if (state.operation == 'record') {
  				add_url_event = true;
  				chrome.webNavigation.onCompleted.addListener(function(completedDetails) {
              chrome.tabs.get(completedDetails.tabId, function (tab) {

      					if (add_url_event) {
      						addNewUrlActionByURL(completedDetails.url, tab.title, true);

      						add_url_event = false;
      					}
            });
  				});
  			}
  		});
  	}
  });
}

/***************** Recording functions **************/
function removeRecordingListeners()
{
  if (chrome.webRequest != null) {
	  if (chrome.webRequest.onBeforeRequest != null && onBeforeRequest != null)
		chrome.webRequest.onBeforeRequest.removeListener(onBeforeRequest);

	  if (chrome.webRequest.onBeforeRedirect != null && onBeforeRedirect != null)
		chrome.webRequest.onBeforeRedirect.removeListener(onBeforeRedirect);

	  if (chrome.webRequest.onCompleted != null && onCompleted != null)
		chrome.webRequest.onCompleted.removeListener(onCompleted);
  }

  if (chrome.tabs.onUpdated != null && urlChange != null)
	chrome.tabs.onUpdated.removeListener(urlChange);

  if (chrome.tabs.onActivated != null && tabActiviated != null)
	chrome.tabs.onActivated.removeListener(tabActiviated);

  removeDebuggers();
}

function addRecordingListeners()
{
  chrome.tabs.onUpdated.addListener(urlChange);
  // chrome.tabs.onCreated.addListener(tabCreated);
  chrome.tabs.onActivated.addListener(tabActiviated);


  if (chrome.webRequest.onBeforeRequest != null)
	chrome.webRequest.onBeforeRequest.addListener(onBeforeRequest, { urls: ['http://*/*', 'https://*/*'], types: ['main_frame', 'sub_frame', 'xmlhttprequest'] }, ["requestBody"]);

  if (chrome.webRequest.onBeforeRedirect != null)
	chrome.webRequest.onBeforeRedirect.addListener(onBeforeRedirect,  { urls: ['http://*/*', 'https://*/*'], types: ['main_frame', 'sub_frame', 'xmlhttprequest'] }, ['responseHeaders'] );

  if (chrome.webRequest.onCompleted != null)
	chrome.webRequest.onCompleted.addListener(onCompleted, { urls: ['http://*/*', 'https://*/*'], types: ['main_frame', 'sub_frame', 'xmlhttprequest'] } );
}

var urlList = [];
function urlChange(tabId, details, tab) {
   if (details.status == "loading" && details.url != undefined) {
     urlList.push(details);
   } else if (details.status == "complete") {
       if (urlList.length >= 1) {
        var curEvent = urlList[urlList.length - 1];

        var code = CodeObject.ConstructObject(Code_Actions.Event_New_URL, "", curEvent.url);

         chrome.tabs.sendRequest(tabId, {method: "getScreenData"}, function(response) {
             if(response.method=="getScreenData"){
               var img = response.data;
               var action_event = ActionObject.ConstructObject("URL Change Event", "URL Change Event", "", curEvent.url, tab.title, getTime(), code, img, null, null, null, null, null);
               action_event.type = "EventAction";

                   addEventAction(action_event);

               urlList=[];
             }
         });
     }
  }
}

function tabCreated(tabEvent) {
    var code = CodeObject.ConstructObject(Code_Actions.Event_New_Tab, tabEvent.title, tabEvent.url);

     chrome.tabs.sendRequest(tabEvent.openerTabId, {method: "getScreenData"}, function(response) {
         if(response.method=="getScreenData"){
           var img = response.data;
           var action_event = ActionObject.ConstructObject("New Tab Event", "New Tab Event", "", tabEvent.url, tabEvent.title, getTime(), code, img, null, null, null, null, null);
           action_event.type = "EventAction";

           addEventAction(action_event);
         }
     });
}

function tabActiviated(activeInfo) {
  chrome.tabs.get(activeInfo.tabId, function (tab) {
    var code = CodeObject.ConstructObject(Code_Actions.Event_Switch_Tab, tab.title, tab.url);

     chrome.tabs.sendRequest(activeInfo.tabId, {method: "getScreenData"}, function(response) {
         if(response == undefined){
           var action_event = ActionObject.ConstructObject("Switch Tab Event", "Switch Tab Event", "", tab.url, tab.title, getTime(), code, null, null, null, null, null, null);
           action_event.type = "EventAction";

           addEventAction(action_event);

         } else if (response.method=="getScreenData") {
           var img = response.data;
           var action_event = ActionObject.ConstructObject("Switch Tab Event", "Switch Tab Event", "", tab.url, tab.title, getTime(), code, img, null, null, null, null, null);
           action_event.type = "EventAction";

           addEventAction(action_event);
         }
     });
  });
}
