const Code_Actions = {
	Open_URL: 'Open URL',
	Click_At: 'Click At',
	Click_XPath: 'Click XPath',
	Click_Id: 'Click Id',
	Click_Name: 'Click Name',
	Click_ClassName: 'Click Class',
	Custom_Code: 'Custom',
	DoubleClick_At: 'Double Click At',
	DragAndDropToObject: 'Drag and Drop To Object',
	Select_By_Index: 'Select By Index',
	Submit_XPath: 'Submit XPath',
	Submit_ClassName : 'Submit By Class',
	Submit_Id: 'Submit By Id',
	Submit_Name: 'Submit By Name',
	Select_Window: 'Select Window',
	Select_Frame: 'Select Frame',
	Send_Keys: 'Send Keys',
	Enter_Search: 'Enter Search',
	Pause: 'Pause',
	WaitForElementPresent: 'Wait for Element Present',
	WaitForPageToLoad: 'Wait for Page Load',
	AssertText: 'Assert Text',
	AssertTitle: 'Assert Title',
	AssertValue: 'Assert Value',
	Comment: 'Comment',
	Event_New_URL: 'URL Change',
	Event_New_Tab: 'New Tab',
	Event_Switch_Tab: 'Switch Tab',
	END_URL: 'End URL'
};

const Action_Type = {
	Click: 'Click',
	EnterText: 'EnterText',
	Other: 'Other'
};


const Code_Builders = {
	Selenium_Java: 'Selenium Java',
	Selenium: 'Selenium JavaScript',
	JSON: 'JSON (Native)'
};

const Key_Press_Type = {
	Key_Enter : '${KEY_ENTER}',
	Key_Up : '${KEY_UP}',
	Key_Down : '${KEY_DOWN}',
	Key_Left : '${KEY_LEFT}',
	Key_Right : '${KEY_RIGHT}',
	Key_Tab : '${KEY_TAB}'
}

var codeBuilder = {
	// Code types - text, radio, checkbox, file, submit, image, range, reset, select, a, element
	buildClickCode : function(cur_element, classifier_hash, x_path)
	{
		if (classifier_hash == null)
			return null;

		switch (classifier_hash.type) {
			case "text":
			case "radio":
			case "element":
			case "image":
			case "range":
			case "reset":
			case "button":
			case "checkbox":
			case "a":
				return CodeObject.ConstructObject(Code_Actions.Click_XPath, x_path, "");
			case "select":
				return CodeObject.ConstructObject(Code_Actions.Click_XPath, x_path, "");
			case "submit":
				return CodeObject.ConstructObject(Code_Actions.Submit_XPath, x_path, "");
			default:
				return CodeObject.ConstructObject(Code_Actions.Comment);
		}
	},

	buildSelectCode : function(cur_element, classifier_hash, xpath, selectedIndex)
	{
		if (classifier_hash == null)
			return null;

		switch (classifier_hash.type) {
			case "select":
				return CodeObject.ConstructObject(Code_Actions.Select_By_Index, xpath, selectedIndex);
			default:
				return null;
		}
	},

	buildSubmitCode : function(cur_element, classifier_hash, x_path)
	{
		if (classifier_hash == null)
			return null;

		switch (classifier_hash.type) {
			case "form":
				return CodeObject.ConstructObject(Code_Actions.Submit_XPath, x_path);
			default:
				return null;
		}
	},

	buildEnterSearchCode : function (cur_element, classifier_hash, x_path)
	{
		if (classifier_hash == null)
			return null;

		switch (classifier_hash.type) {
			case "text":
				return CodeObject.ConstructObject(Code_Actions.Enter_Search, x_path);
			default:
				return null;
		}
	},

	buildKeyPressCode : function(cur_element, classifier_hash, x_path)
	{
		if (classifier_hash == null)
			return null;

		switch (classifier_hash.type) {
			case "text":
				return CodeObject.ConstructObject(Code_Actions.Send_Keys, x_path, cur_element.value);
			default:
				return null;
		}
	},

	buildKeyDownCode : function(x_path, value)
	{
		return CodeObject.ConstructObject(Code_Actions.Send_Keys, x_path, value);
	},

	buildDragCode : function (cur_element, classifier_hash, item_path, drop_path)
	{
		if (classifier_hash == null)
			return null;

		return CodeObject.ConstructObject(Code_Actions.DragAndDropToObject, item_path, drop_path);
	},

	createCode : function(codeType, settings_header, settings_footer, action_list)
	{
		if (codeType == Code_Builders.Selenium) {
			return seleniumWebdriverJS.create_selenium2_code(settings_header, settings_footer, action_list);
		} else if (codeType == Code_Builders.JSON) {
			return jsonBuilder.create_json_code(action_list);
		} else if (codeType == Code_Builders.Selenium_Java) {
			return seleniumWebdriverJava.create_code(settings_header, settings_footer, action_list);
		}
	},

	getExportExtension : function (codeType)
	{
		if (codeType == Code_Builders.Selenium) {
			return seleniumWebdriverJS.getExportExtension();
		} else if (codeType == Code_Builders.JSON) {
			return jsonBuilder.getExportExtension();
		} else if (codeType == Code_Builders.Selenium_Java) {
			return seleniumWebdriverJava.getExportExtension();
		}
	},

	getDefaultScriptSettings : function ()
	{
		return {
			Selenium_Java: {
				settings_header: seleniumWebdriverJava.default_header,
				settings_footer: seleniumWebdriverJava.default_footer
			},
			Selenium: {
				settings_header: seleniumWebdriverJS.default_header,
				settings_footer: seleniumWebdriverJS.default_footer
			},
			JSON: {
				settings_header: jsonBuilder.default_header,
				settings_footer: jsonBuilder.default_footer
			}
		};
	}
};
