var seleniumWebdriverJava = {
	default_header:
		"import org.openqa.selenium.By; \n" +
		"import org.openqa.selenium.WebElement; \n" +
		"import org.junit.Assert; \n" +
		"import org.openqa.selenium.Keys; \n" +
		"import org.openqa.selenium.support.ui.Select; \n" +
		"import org.openqa.selenium.chrome.ChromeOptions; \n" +
		"import org.openqa.selenium.chrome.ChromeDriver; \n \n" +
		"public class Script { \n" +
		"\tpublic static void main(String[] args){ \n" +
		"\t\tChromeOptions options = new ChromeOptions(); \n" +
		'\t\toptions.addArguments("--start-maximized"); \n' +
        "\t\tChromeDriver driver=new ChromeDriver(options);\n\n",


		default_footer: "\t} \n" +
						"}",

	// Function to concatenate code together
	create_code : function (settings_header, settings_footer, action_list)
	{
		// construct code
		var total_code = "";

		if (settings_header != null) {
			total_code = total_code + "// Header \n";
			total_code = total_code + settings_header + "\n";
		}

		for (i = 0; i < action_list.length; i++) {
			total_code = total_code + "\t\t//" + action_list[i].english.trim().replace(/(\r\n|\n|\r)/gm,"") + "\n";

			total_code = total_code + "\t\t" + seleniumWebdriverJava.create_snippet(action_list[i]) + "\n \n";
		}

		if (settings_footer != null) {
			total_code = total_code + "\t// Footer \n";
			total_code = total_code + settings_footer + "\n";
		}

		return total_code;
	},

	// Function to return line of code
	create_snippet: function (action_item)
	{
		if (action_item.code.type == Code_Actions.Open_URL) {
			return 'driver.get("' + action_item.code.value + '");';
		} else if (action_item.code.type == Code_Actions.Click_At) {
			 return 'driver.clickAt(By.xpath("' + action_item.code.elem + '",' + action_item.code.value +  '));';
		} else if (action_item.code.type == Code_Actions.Click_XPath) {
			 return 'driver.findElement(By.xpath("' + action_item.code.elem + '")).click();';
		} else if (action_item.code.type == Code_Actions.Click_Id) {
			 return 'driver.findElement(By.id("' + action_item.code.elem + '")).click();';
		} else if (action_item.code.type == Code_Actions.Click_Name) {
			 return 'driver.findElement(By.name("' + action_item.code.elem + '")).click();'
		} else if (action_item.code.type == Code_Actions.Click_ClassName) {
			 return 'driver.findElement(By.class("' + action_item.code.elem + '")).click();';
		} else if (action_item.code.type == Code_Actions.Custom_Code) {
			return action_item.code.value;
		} else if (action_item.code.type == Code_Actions.Enter_Search) {
			return 'driver.findElement(By.xpath("' + action_item.code.elem + '")).sendKeys("Keys.RETURN");';
		} else if (action_item.code.type == Code_Actions.DoubleClick_At) {
			 return 'driver.doubleClickAt(By.xpath("' + action_item.code.elem + '",' + action_item.code.value +  '));';
		} else if (action_item.code.type == Code_Actions.DragAndDropToObject) {
			 return 'driver.dragAndDrop(By.xpath(' + action_item.code.elem + '), By.xpath(' + action_item.code.value + '));'
		} else if (action_item.code.type == Code_Actions.Submit_XPath) {
			 return 'driver.findElement(By.xpath("' + action_item.code.elem + '")).submit();';
		} else if (action_item.code.type == Code_Actions.Submit_ClassName) {
			 return 'driver.findElement(By.class("' + action_item.code.elem + '")).submit();';
		} else if (action_item.code.type == Code_Actions.Submit_Id) {
			 return 'driver.findElement(By.id("' + action_item.code.elem + '")).submit();';
		} else if (action_item.code.type == Code_Actions.Select_By_Index) {
			 return 'new Select(driver.findElement(By.id("' + action_item.code.elem + '"))).selectByIndex(' + action_item.code.value + ');';
		} else if (action_item.code.type == Code_Actions.Submit_Name) {
			 return 'driver.findElement(By.name("' + action_item.code.elem + '")).submit();';
		} else if (action_item.code.type == Code_Actions.Select_Window) {
			return 'driver.selectWindow("' + action_item.code.elem + '");';
		} else if (action_item.code.type == Code_Actions.Select_Frame) {
			return 'driver.selectFrame("' + action_item.code.elem + '");';
		} else if (action_item.code.type == Code_Actions.Send_Keys) {
			// Replace enter keys with keys.return
			if (action_item.code.value == Key_Press_Type.Key_Enter) {
				return 'driver.findElement(By.xpath("' + action_item.code.elem + '")).sendKeys(Keys.RETURN);';
			} else if (action_item.code.value == Key_Press_Type.Key_Up) {
				return 'driver.findElement(By.xpath("' + action_item.code.elem + '")).sendKeys(Keys.ARROW_UP);';
			} else if (action_item.code.value == Key_Press_Type.Key_Down) {
				return 'driver.findElement(By.xpath("' + action_item.code.elem + '")).sendKeys(Keys.ARROW_DOWN);';
			} else if (action_item.code.value == Key_Press_Type.Key_Right) {
				return 'driver.findElement(By.xpath("' + action_item.code.elem + '")).sendKeys(Keys.ARROW_RIGHT);';
			} else if (action_item.code.value == Key_Press_Type.Key_Left) {
				return 'driver.findElement(By.xpath("' + action_item.code.elem + '")).sendKeys(Keys.ARROW_LEFT);';
			} else if (action_item.code.value == Key_Press_Type.Key_Tab) {
				return 'driver.findElement(By.xpath("' + action_item.code.elem + '")).sendKeys(Keys.TAB);';
			} else {
				return 'driver.findElement(By.xpath("' + action_item.code.elem + '")).sendKeys("' + action_item.code.value + '");';
			}
		} else if (action_item.code.type == Code_Actions.Pause) {
			return 'driver.sleep(' + action_item.code.value + ');';
		} else if (action_item.code.type == Code_Actions.WaitForElementPresent) {
			return 'driver.wait(until.elementLocated(By.id("lst-ib")));';
		} else if (action_item.code.type == Code_Actions.WaitForPageToLoad) {
			return 'driver.waitForPageToLoad(' + action_item.code.value + ');';
		} else if (action_item.code.type == Code_Actions.Comment) {
			return '// ' + action_item.code.value.replace(/(\r\n|\n|\r)/gm,'');
		} else if (action_item.code.type == Code_Actions.AssertText) {
			return 'driver.findElement(By.xpath("' + action_item.code.elem + '"),10).getText() .then(function(textValue) { Assert.equal(inputValue,"' + action_item.code.value + '");});';
		} else if (action_item.code.type == Code_Actions.AssertTitle) {
			return 'driver.getTitle().then(function(title) { Assert.equal(inputValue,"' + action_item.code.value + '");} );';
		} else if (action_item.code.type == Code_Actions.AssertValue) {
			return 'driver.findElement(By.xpath("' + action_item.code.elem + '"),10).getAttribute("value")).then(function(inputValue) { Assert.equal(inputValue,"' + action_item.code.value + ' ");} );';
		} else {
			return '';
		}

		return '';
	},

	getExportExtension : function () {
		return 'java';
	}
};
