var seleniumWebdriverJS = {
	default_header: 
		"var webdriver = require('selenium-webdriver'); \n" + 
		"var until = require('selenium-webdriver/lib/until'); \n" + 
		"var assert = require('assert'); \n" +
		"var By = webdriver.By; \n" +
		"var driver = new webdriver.Builder() \n" + 
		".forBrowser('chrome') \n" + 
		".setChromeOptions( /* … */) \n" + 
		".build() \n",
	
	default_footer: "driver.quit()",

	// Function to concatinate code toghether
	create_selenium2_code : function (settings_header, settings_footer, action_list)
	{
		// construct code
		var total_code = "";
		
		if (settings_header != null) {
			total_code = total_code + "// Header \n";
			total_code = total_code + settings_header + "\n";
		}

		for (i = 0; i < action_list.length; i++) {
			total_code = total_code + "//" + action_list[i].english.trim().replace(/(\r\n|\n|\r)/gm,"") + "\n";
			
			total_code = total_code + seleniumWebdriverJS.create_selenium2_snippet(action_list[i]) + "\n \n";
		}
		
		if (settings_footer != null) {
			total_code = total_code + "// Footer \n";
			total_code = total_code + settings_footer + "\n";
		}
				
		return total_code;		
	},

	// Function to return line of code
	create_selenium2_snippet: function (action_item)
	{
		if (action_item.code.elem) {
			return "driver.wait(until.elementLocated(webdriver.By.name('" + action_item.code.elem + "'))) \n" + 
				    seleniumWebdriverJS.create_code_for_action(action_item);
		} else {
			return seleniumWebdriverJS.create_code_for_action(action_item);
		}
	},
	
	create_code_for_action: function (action_item) 
	{
		if (action_item.code.type == Code_Actions.Open_URL) {
			return "driver.get('" + action_item.code.value + "');";		
		} else if (action_item.code.type == Code_Actions.Click_At) {
			 return "driver.clickAt(By.xpath('" + action_item.code.elem + "'," + action_item.code.value +  "));";		
		} else if (action_item.code.type == Code_Actions.Click_XPath) {
			 return "driver.findElement(By.xpath('" + action_item.code.elem + "')).click();";		
		} else if (action_item.code.type == Code_Actions.Click_Id) {
			 return "driver.findElement(By.id('" + action_item.code.elem + "')).click();";				
		} else if (action_item.code.type == Code_Actions.Click_Name) {
			 return "driver.findElement(By.name('" + action_item.code.elem + "')).click();"						
		} else if (action_item.code.type == Code_Actions.Click_ClassName) {
			 return "driver.findElement(By.class('" + action_item.code.elem + "')).click();";				
		} else if (action_item.code.type == Code_Actions.Custom_Code) {
			return action_item.code.value;
		} else if (action_item.code.type == Code_Actions.Enter_Search) {
			return "driver.findElement(By.xpath('" + action_item.code.elem + "')).sendKeys('${KEY_ENTER}');";					
		} else if (action_item.code.type == Code_Actions.DoubleClick_At) {
			 return "driver.doubleClickAt(By.xpath('" + action_item.code.elem + "'," + action_item.code.value +  "));";		
		} else if (action_item.code.type == Code_Actions.DragAndDropToObject) {
			 return "driver.dragAndDrop(By.xpath(" + action_item.code.elem + "), By.xpath(" + action_item.code.value + "));"
		} else if (action_item.code.type == Code_Actions.Submit_XPath) {
			 return "driver.findElement(By.xpath('" + action_item.code.elem + "')).submit();";				
		} else if (action_item.code.type == Code_Actions.Submit_ClassName) {
			 return "driver.findElement(By.class('" + action_item.code.elem + "')).submit();";						
		} else if (action_item.code.type == Code_Actions.Submit_Id) {
			 return "driver.findElement(By.id('" + action_item.code.elem + "')).submit();";				
		} else if (action_item.code.type == Code_Actions.Select_By_Index) {
			 return "driver.findElement(By.id('" + action_item.code.elem + "')).selectByIndex('" + action_item.code.value + "');";
		} else if (action_item.code.type == Code_Actions.Submit_Name) {
			 return "driver.findElement(By.name('" + action_item.code.elem + "')).submit();";								
		} else if (action_item.code.type == Code_Actions.Select_Window) {
			return "driver.selectWindow('" + action_item.code.elem + "');";
		} else if (action_item.code.type == Code_Actions.Select_Frame) {
			return "driver.selectFrame('" + action_item.code.elem + "');";
		} else if (action_item.code.type == Code_Actions.Send_Keys) {
			return "driver.findElement(By.xpath('" + action_item.code.elem + "')).sendKeys('" + action_item.code.value + "');";		
		} else if (action_item.code.type == Code_Actions.Pause) {
			return "driver.sleep(" + action_item.code.value + ");";
		} else if (action_item.code.type == Code_Actions.WaitForElementPresent) {
			return "driver.wait(until.elementLocated(webdriver.By.id('lst-ib')));";
		} else if (action_item.code.type == Code_Actions.WaitForPageToLoad) {
			return "driver.waitForPageToLoad(" + action_item.code.value + ");";
		} else if (action_item.code.type == Code_Actions.Comment) {
			return "// " + action_item.code.value.replace(/(\r\n|\n|\r)/gm,"");
		} else if (action_item.code.type == Code_Actions.AssertText) {
			return "driver.findElement(By.xpath('" + action_item.code.elem + "'),10).getText() .then(function(textValue) { assert.equal(inputValue,'" + action_item.code.value + "');});";		
		} else if (action_item.code.type == Code_Actions.AssertTitle) {
			return "driver.getTitle().then(function(title) { assert.equal(inputValue,'" + action_item.code.value + "');} );";		
		} else if (action_item.code.type == Code_Actions.AssertValue) {
			return "driver.findElement(By.xpath('" + action_item.code.elem + "'),10).getAttribute('value')).then(function(inputValue) { assert.equal(inputValue,'" + action_item.code.value + " ');} );";
		} else {
			return "";
		}
		
		return "";	
	},
	
	getExportExtension : function () {
		return 'js';
	}	
};