
/* content script embeded in webpage */
/* Does the actual recording */

// On a page load send signal to background
chrome.runtime.sendMessage({ operation: 'load' });

var strategy_list = [SelectorStrategies.Id, SelectorStrategies.Name, SelectorStrategies.Class, SelectorStrategies.Tag, SelectorStrategies.X_Path, SelectorStrategies.ServiceNowName, SelectorStrategies.ServiceNowId];

var preventClick = false;
var preventType = false;

window.scanning = true;

var recording = false;

var shifted = false;
var ctrled = false;

var paused = false;

var identifierMappings = [];
var identifierPreferences = [];
var identifierAttributes = [];

var selectorType = null;
var customScan = null;
var searchDepth = 4;
var objectIndex = false;


$(document).on('keydown', function(e){
  if (recording)
    return;

  if(e.key === "Shift") {
    shifted = !shifted;

    if (!paused) {
      paused = true;

      chrome.runtime.sendMessage({ operation: 'pause'});
    } else {
      paused = false;

      chrome.runtime.sendMessage({ operation: 'resume_scan'});
    }
  } else if (e.key === "Control") {
    ctrled = true;
  }
});

$(document).on('keyup', function(e){
    ctrled = false;
});

// Get current time
function getTime() {
  return new Date().getTime();
}


// Called when a change happens to the browser
function recordChange(event) {
	if (document.readyState != "complete")
		return;

	if (preventType)
		return;

  var curTarget = getElementFromEvent(event);

	var action_type = classifier(curTarget);
	if (action_type == null)
		return;

	var action_event = null;
	if (action_type.type == 'select') {

		action_event  = Recorder.constructSelectEvent(event, action_type, curTarget, globalSelector.getUniqueSelector(curTarget, strategy_list, Action_Type.Other, customScan), Action_Type.Other, customScan);
	} else {
		action_event  = Recorder.constructTypeEvent(event, action_type, curTarget, globalSelector.getUniqueSelector(curTarget, strategy_list, Action_Type.EnterText, customScan), Action_Type.EnterText, customScan);
	}

	if (action_event == null)
		return;

	chrome.runtime.sendMessage({ operation: 'action', action: action_event });
}

// Called when a click happens inside the browser
function recordClick(event) {
	if (document.readyState != "complete")
		return;

	if (preventClick)
		return;

  var curTarget = getElementFromEvent(event);

	var action_type = classifier(curTarget);

	if (action_type == null)
		return;

	if (curTarget.tagName.toLowerCase() == 'select')
		return;

	var action_event  = Recorder.constructClickEvent(event, action_type, curTarget, globalSelector.getUniqueSelector(curTarget, strategy_list, Action_Type.Click, customScan), Action_Type.Click, customScan);

	if (action_event == null)
		return;

	chrome.runtime.sendMessage({ operation: 'action', action: action_event });
}

function scanClick(event) {
  if (!shifted) {
    event.stopImmediatePropagation();
    event.stopPropagation();
    event.preventDefault();

    if (!ctrled) {
      var curTarget = getElementFromEvent(event);

      scanElement(curTarget);
    } else {
      ctrled = false;

      // links
      var curTarget = getElementFromEvent(event);
      var links = curTarget.querySelectorAll('a');
      for (var item of links) {
          try {
            scanElement(item);
          } catch (err) {}
      }

      // input
      var inputElements = curTarget.querySelectorAll('input');
      for (var item of inputElements) {
          try {
            scanElement(item);
          } catch (err) {}
      }

      // select
      var selectOptions = curTarget.querySelectorAll('select');
      for (var item of selectOptions) {
          try {
            scanElement(item);
          } catch (err) {}
      }

      // button
      var buttons = curTarget.querySelectorAll('button');
      for (var item of buttons) {
          try {
            scanElement(item);
          } catch (err) {}
      }
    }
  }
}

function scanElement(curTarget) {
  // Todo go up 5 layers;
  // ScannedObjects =  {{TagName: '', Object: {}}
  var initialTarget = curTarget;

  var initalObject = buildObject(curTarget, true);

  var scannedObjects = [{tagName: curTarget.tagName, index: 0,  obj: initalObject}];

  var i = 0;
  while (curTarget.parentNode != null) {
      if (i >= 5)
        break;

      curTarget = curTarget.parentNode;

      var curObj = buildObject(curTarget, false);

      if (curObj == null)
        break;

      scannedObjects.push({tagName: curTarget.tagName, index:(i+1), obj: curObj});

      i++;
  }

  if (initalObject != null) {

    // Bubble up to a button or link or input
    var defaultObjId = 0;
    for (var i = 0; i < scannedObjects.length; i++) {
      if (scannedObjects[i].tagName.toLowerCase() === "a" || scannedObjects[i].tagName.toLowerCase() === "button" || scannedObjects[i].tagName.toLowerCase() === "input") {
        defaultObjId = i;

        break;
      }
    }

    initialTarget.style.setProperty("background-color", "red", "important");

    chrome.runtime.sendMessage({ operation: 'scraped_object', object: {currentObject: scannedObjects[defaultObjId], scannedObjects: scannedObjects }} );
  }
}

function buildObject(curTarget, expand) {
    if (curTarget.tagName == null)
      return null;

    var shadowParent = getNodeShadowParent(curTarget);

    screenScraper.identifierPreferences = identifierPreferences;
    screenScraper.identifierMaps = identifierMappings;
    screenScraper.identifierAttrs = identifierAttributes;
    screenScraper.selectorType = selectorType;
    screenScraper.searchDepth = searchDepth;
    screenScraper.customScan = customScan;
    screenScraper.positionIndex = objectIndex;

    var obj = screenScraper.buildIdentifierFromDomRecursive(curTarget, customScan, shadowParent, expand);
	obj.name = obj.name + "";

    if (obj != null)
      obj.shadowRoot = getShadowRoot(curTarget);

    return obj;
}

function isInShadow(node) {
    for (; node; node = node.parentNode) {
        if (node.toString() === "[object ShadowRoot]") {
            return true;
        }
    }
    return false;
}

function getNodeShadowParent(node)
{
  var parentIdentifier = null;

	var isEleInShadow = isInShadow(node);

	if (isEleInShadow) {
    parentIdentifiers = '';

		var counter = 0;
		while(counter < 100000) {
			if (node === null) {
        return parentIdentifier;
      }

			if (node.toString() === "[object ShadowRoot]") {
        return node.host;
			} else {
				node = node.parentNode;
			}

			counter++;
		}
	}

  return parentIdentifier;
}

function getShadowRoot(node)
{
  var parentIdentifiers = null;

	var isEleInShadow = isInShadow(node);

  if (isEleInShadow) {
    parentIdentifiers = [];

		var counter = 0;
		while(counter < 100000) {
			if (node === null) {
        break;
      }

			if (node.toString() === "[object ShadowRoot]") {
        parentIdentifiers.push(node.host);

				node = node.host;
			} else {
				node = node.parentNode;
			}

			counter++;
		}
	}

  // Starting at lowest -> highest
  var returnIdentifier = null;
  if (parentIdentifiers != null) {
    var returnIdentifier = '';

    for (var i = 0; i < parentIdentifiers.length; i++) {
      var parentElem = null;

      if (i + 1 < parentIdentifiers.length) {
        parentElem = parentIdentifiers[i+1];
      }

      returnIdentifier = "[" + screenScraper.buildShortCssPathShadowDom(parentIdentifiers[i], parentElem) + "]" + (returnIdentifier !== '' ? ">" : "") + returnIdentifier;
    }
  }

  return returnIdentifier;
}


function cancelEvent(event) {
  if (!shifted) {
    try {
      event.stopImmediatePropagation();
      event.stopPropagation();
      // event.preventDefault();
    } catch (e) {
      console.log(e);
    }
  }

}

function scanChange(event) {
  if (document.readyState != "complete")
    return;

  if (!shifted) {
    var curTarget = getElementFromEvent(event);
    var initialTarget = curTarget;

    var initalObject = buildObject(curTarget, true);


    var scannedObjects = [{tagName: curTarget.tagName, index: 0,  obj: initalObject}];

    // Sukgu - Don't trace up the hierarchy
    if (customScan != "Sukgu") {
      var i = 0;
      while (curTarget.parentNode != null) {
          if (i >= 5)
            break;

          curTarget = curTarget.parentNode;

          var curObj = buildObject(curTarget, false);

          if (curObj == null)
            break;

          scannedObjects.push({tagName: curTarget.tagName, index:(i+1), obj: curObj});

          i++;
      }
    }

    if (initalObject != null) {
      initialTarget.style.setProperty("background-color", "red", "important");

      chrome.runtime.sendMessage({ operation: 'scraped_object', object: { currentObject: initalObject, scannedObjects: scannedObjects } } );


      event.preventDefault();
      event.stopPropagation();
    }

  }
}

var start_element;
// Gives us location of the element being moved
function dragStartOccured(event) {
  var curTarget = getElementFromEvent(event);

	start_element = curTarget;
}

function dragEndOccured(event) {
	if (start_element == null)
		return;

	if (document.readyState != "complete")
		return;

  var curTarget = getElementFromEvent(event);

	var action_type = classifier(curTarget);

	if (action_type == null)
		return;

	var action_event = Recorder.constructDragEvent(event, action_type, start_element, globalSelector.getUniqueSelector(start_element, strategy_list, Action_Type.Other, customScan), curTarget, globalSelector.getUniqueSelector(curTarget, strategy_list), customScan);

	start_element = null;

	if (action_event == null)
		return;

	chrome.runtime.sendMessage({ operation: 'action', action: action_event });
}

function getFormElement(element)
{
	var curElement = element;
	var tagName = curElement.tagName.toLowerCase();

	while ((tagName != 'form') && (tagName != 'body')) {
		curElement = curElement.parentElement;

		if (curElement == null)
			return null;

		tagName = curElement.tagName.toLowerCase();
	}

	if (tagName != 'form')
		return null;

	if (!curElement.hasAttribute("onsubmit"))
		return null;

	return curElement;
}

function getElementFromEvent(event)
{
  if (event.composedPath() != null && event.composedPath().length > 0) {
    for (var i = 0; i < event.composedPath().length; i++) {
      if (!isHidden(event.composedPath()[i])) {
        return event.composedPath()[i];
      }
    }

    return event.composedPath()[0];
  }

  return event.target;
}

function isHidden(el) {
    var style = window.getComputedStyle(el);
    return (style.display === 'none')
}

// Key is pressed
function keydown(event) {
  var curTarget = getElementFromEvent(event);

	// Only handle key events on tags
	if (!curTarget.tagName)
		return;

	var key = event.keyCode;
	var tagName = curTarget.tagName.toLowerCase();
	var type = curTarget.type;

	// If type is not input then return (we only handle keys for user input)
	if (tagName != 'input')
		return;

	var key_code = "";
	if (key == 13) { // Enter key
		key_code = Key_Press_Type.Key_Enter;
	} else if (key == 38) { // Up key
		key_code = Key_Press_Type.Key_Up;
	} else if (key == 39) { // Right key
		key_code = Key_Press_Type.Key_Right;
	} else if (key == 40) { // Down key
		key_code = Key_Press_Type.Key_Down;
	} else if (key == 9) { // Tab key
		// key_code = Key_Press_Type.Key_Tab;
    // Ignore TABS
    return;
	} else {
		return;
	}

	var action_type = classifier(curTarget);
	if (action_type == null)
		return;

	// Simulate 'type' event
	var type_event  = Recorder.constructTypeEvent(event, action_type, curTarget, globalSelector.getUniqueSelector(curTarget, strategy_list, Action_Type.Other, customScan), Action_Type.Other, customScan);

	if (type_event == null)
		return;

	chrome.runtime.sendMessage({ operation: 'action', action: type_event});

	var key_enter_event = null;
	var form_element = getFormElement(curTarget);

	if (key_code == Key_Press_Type.Key_Enter && (form_element != null)) { // Enter key && form
		// Submit
		action_type = classifier(form_element);
		if (action_type == null)
			return;

		key_enter_event = Recorder.constructSubmitEvent(event, action_type, form_element, globalSelector.getUniqueSelector(form_element, strategy_list, Action_Type.Other, customScan), Action_Type.Other, customScan);
	} else {
		// Simulate 'key' event
		key_enter_event = Recorder.constructKeyPressEvent(event, action_type, curTarget, globalSelector.getUniqueSelector(curTarget, strategy_list, Action_Type.Other, customScan), key_code, Action_Type.Other, customScan);
	}

	if (key_enter_event == null)
		return;
	chrome.runtime.sendMessage({ operation: 'action', action: key_enter_event });

	// Prevent click & type events
	preventClick = true;
	preventType = true;
	setTimeout(function() {
		preventClick = false;
	}, 500);
}

  $(window).on('mouseenter', function (e) {
    if (window.scanning) {
      $(e.target).addClass("el-selection");
    }
  });

  $(window).on('mouseleave', function (e) {
    if (window.scanning) {
      $(e.target).removeClass("el-selection");
    }
  });

// Listen to messages sent from background.js
chrome.runtime.onMessage.addListener(
  (request, sender, sendResponse) => {
  	if (request.operation === 'record') {
      shifted = false;

      paused = false;

      window.scanning = false;

      recording = true;

  		document.addEventListener('keydown', keydown, true);
  		document.addEventListener('change', recordChange, true);
  		document.addEventListener('click', recordClick, true);
  		document.addEventListener('drag', dragStartOccured, true);
  		document.addEventListener('drop', dragEndOccured, true);


  	} else if (request.operation === 'stop') {
      shifted = false;

      paused = true;

      window.scanning = false;

      recording = false;

  		document.removeEventListener('keydown', keydown, true);
	    document.removeEventListener('change', recordChange, true);
  		document.removeEventListener('click', recordClick, true);
      document.removeEventListener('click', scanClick, true);
      document.removeEventListener('touchstart', recordClick, true);
      document.removeEventListener('touchstart', scanClick, true);
      document.removeEventListener('change', scanChange, true);
  		document.removeEventListener('drag', dragStartOccured, true);
  		document.removeEventListener('drop', dragEndOccured, true);

      document.removeEventListener('focus', cancelEvent, true);
      document.removeEventListener('focusin', cancelEvent, true);
      document.removeEventListener('touchstart', cancelEvent, true);
      document.removeEventListener('mousedown', cancelEvent, true);
      document.removeEventListener('focusout', cancelEvent, true);
      document.removeEventListener('touchend', cancelEvent, true);
      document.removeEventListener('mouseup', cancelEvent, true);
      document.removeEventListener('mouseover', cancelEvent, true);
      document.removeEventListener('pointerover', cancelEvent, true);
      document.removeEventListener('pointerenter', cancelEvent, true);
      document.removeEventListener('pointerdown', cancelEvent, true);


      $(this).off('touchstart click');

    } else if (request.operation === 'record_scanner') {
      shifted = false;

      paused = false;

      window.scanning = true;

      document.addEventListener('change', scanChange, true);
      document.addEventListener('touchstart', scanClick,  { passive: false }, true);
      document.addEventListener('click', scanClick, true);
      document.addEventListener('pointerdown', cancelEvent, true);
      document.addEventListener('mousedown', cancelEvent, true);
      document.addEventListener('focusout', cancelEvent, true);
      document.addEventListener('touchend', cancelEvent, true);
      document.addEventListener('mouseup', cancelEvent, true);
      document.addEventListener('pointerover', cancelEvent, true);
      document.addEventListener('pointerenter', cancelEvent, true);
      document.addEventListener('focus', cancelEvent, true);
      document.addEventListener('focusin', cancelEvent, true);
      document.addEventListener('touchstart', cancelEvent, true);

      // document.addEventListener('mouseover', cancelEvent, true);

      selectorType = request.selector_type;

  	} else if (request.operation === 'alert') {
      alert(request.data);
    }

    customScan = request.custom_scan;

    if (request.identifier_mappings != null) {
      identifierMappings = request.identifier_mappings;
    }

    if (request.identifier_preferences != null) {
      identifierPreferences = request.identifier_preferences;
    }

    if (request.identifier_attributes != null) {
      identifierAttributes = request.identifier_attributes;
    }

    if (request.searchDepth != null)
      searchDepth = request.searchDepth;

    if (request.objectIndex != null)
      objectIndex = request.objectIndex;

    return true;
  }
);

if (chrome.extension.onRequest != null)
{
	chrome.extension.onRequest.addListener(
		function(request, sender, sendResponse) {
		  if (request.method === "getScreenData"){
			 sendResponse({data: ImageActionObject.ConstructEmptyObject(), method: "getScreenData"}); //same as innerText

			 return true;
		  } else if (request.operation === "getScreenScrape") {

			  var sData = screenScraper.screenScrape(customScan);

			  console.log(sData);

			  console.log(sender);

			  if (sData.name !== "") {
				   // chrome.runtime.sendMessage({ operation: 'returngetScreenScrape', screendata: sData });
				 sendResponse({screendata: sData, method: "returngetScreenScrape"});
			   }

			 return true;
		  }

		  return true;
		}
	);
}

var inp = document.getElementsByTagName("input");
for (var i = 0; i < inp.length; i++) {
	inp[i].addEventListener("focus", function(event) {
		preventType = false;
	});
}
