const SelectorStrategies = {
	Id: 'Id',
	Class: 'Class',
	Value: 'Value',
	Name: 'Name',
	Tag: 'Tag',
	X_Path: 'XPath',
	TextContent: 'TextContent',
	ServiceNowName: 'ServiceNowName',
	ServiceNowId: 'ServiceNowId'
};

var globalSelector = {
	getObjectId : function(cur_element)
	{
		return cur_element.id;
	},

	getObjectGsft_Id : function(cur_element)
	{
		return cur_element.getAttribute("id");
	},

	getName: function (cur_element)
	{
		return cur_element.name;
	},

	getObjectsForId : function(cur_element, id)
	{
			return document.querySelectorAll('[id=' + CSS.escape(id) + ']');
	},

	getObjectText : function(cur_element)
	{
		return cur_element.textContent;
	},

	getObjectsForText : function(cur_element, text)
	{
			var headings = document.evaluate("count(//*[text()='" + text + "'])", document, null, XPathResult.NUMBER_TYPE, null );

			return headings.numberValue;
	},

	getNoElementsByXPath  : function(xpath)
	{
		try {
			var headings = document.evaluate("count(" + xpath + ")", document, null, XPathResult.NUMBER_TYPE, null );

			return headings.numberValue;
		} catch(err) {
			return 0;
		}
	},

	getNoElementsByCSS  : function(css, shadowParent)
	{
		try {
			var elements;
			if (shadowParent != null && shadowParent.shadowRoot != null) {
				elements = shadowParent.shadowRoot.querySelectorAll(css);
			} else {
				elements = document.querySelectorAll(css);
			}

			return elements.length;

		} catch(err) {
			return 0;
		}
	},

	getElementsByXPath : function(xpath, parent)
	{
	  let results = [];
	  let query = document.evaluate(xpath,
	      parent || document,
	      null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
	  for (let i=0, length=query.snapshotLength; i<length; ++i) {
	    results.push(query.snapshotItem(i));
	  }
	  return results;
	},

	getElementIndex : function (foundElements, element)
	{
		var labelIndex = 0;
		for (var i = 0; i < foundElements.length; i++) {
			if (foundElements[i] == element)
				return labelIndex;

			labelIndex++;
		}

		return 0;
	},

	getUniqueId : function(cur_element)
	{
		var id = globalSelector.getObjectId(cur_element);

		// If its not set return null
		if (id == null || id == '')
			return null;

		// Otherwise check if its unique for identifying the object in the document
		if (globalSelector.getObjectsForId(cur_element, id).length > 1)
			return null;

		//a[@id = 'unique-id']
		return "//" + cur_element.tagName + '[@id="' + id + '"]';
	},

	getObjectName : function(cur_element)
	{
		return cur_element.getAttribute("name");
	},

	getObjectsForName : function(cur_element, name)
	{
		return document.querySelectorAll(cur_element.tagName + '[name="' + name + '"]');
	},

	getUniqueName : function(cur_element)
	{
		var name = globalSelector.getObjectName(cur_element);

		// If its not set return null
		if (!name)
			return null;

		// Otherwise check if its unique for identifying the object in the document
		if (globalSelector.getObjectsForName(cur_element, name).length > 1)
			return null;

		//a["val"]
		return "//" + cur_element.tagName + '[@name="' + name + '"]';
	},

	getObjectValue : function(cur_element)
	{
		return cur_element.value;
	},

	getObjectsForValue : function (cur_element, val)
	{
		return document.querySelectorAll(cur_element.tagName + '[value="' + val + '"]');
	},

	getUniqueValue : function(cur_element)
	{
		var val = globalSelector.getObjectValue(cur_element);

		if (!val)
			return null;

		// Otherwise check if its unique for identifying the object in the document
		if (globalSelector.getObjectsForValue(cur_element, val).length > 1)
			return null;

		//a["val"]
		return "//" + cur_element.tagName + '["' + val + '"]';
	},

	getObjectClass : function(cur_element)
	{
		return cur_element.className;
	},

	getObjectsForClass : function (cur_element, class_id)
	{
			return document.querySelectorAll('.' + class_id);
	},

	getUniqueClass : function(cur_element)
	{
		var class_id = globalSelector.getObjectClass(cur_element);

		// If its not set return null
		if (class_id == null || class_id == '')
			return null;

		// Otherwise check if its unique for identifying the object in the document
		if (globalSelector.getObjectsForClass(cur_element, class_id).length > 1)
			return null;

		//div[@class="Test"] - follow xpath format
		return "//" + cur_element.tagName + '[@class="' + class_id + '"]';
	},

	getObjectTag : function(cur_element)
	{
		return cur_element.tagName;
	},

	getObjectsForTag : function (cur_element, tagName)
	{
			return document.querySelectorAll(tagName);
	},

	getUniqueTag : function(cur_element)
	{
		var tagName = globalSelector.getObjectTag(cur_element);

		// If its not set return null
		if (tagName == null || tagName == '')
			return null;

		// Otherwise check if its unique for identifying the object in the document
		if (globalSelector.getObjectsForTag(cur_element, tagName).length > 1)
			return null;

		return "//" + tagName;
	},

	getXPath : function(cur_element)
	{
		return Xpath.getElementXPath(cur_element);
	},

	getIdXPath : function(cur_element)
	{
		return Xpath.getElementIdXPath(cur_element);
	},

	getIdCssSelector : function(element)
	{
		if (element && element.id)
				return '#' + element.id;
		else
				return null;
	},

	getAriaLabelXPath : function(cur_element)
	{
		var ariaLabel = cur_element.getAttribute("aria-label");

		// If its not set return null
		if (ariaLabel == null || ariaLabel == '')
			return null;

		// Otherwise check if its unique for identifying the object in the document
		var elements = document.querySelectorAll(cur_element.tagName + '[aria-label=' + CSS.escape(ariaLabel) + ']');
		if (elements.length > 1) {
			// IF more than one found find the element
			for (var i = 0; i < elements.length; i++) {
				if (cur_element == elements[i]) {
					return "(//" + cur_element.tagName + '[@aria-label="' + ariaLabel + '"])[' + (i+1) + ']';
				}
			}
		} else if (elements.length == 1) {
			return "//" + cur_element.tagName + '[@aria-label="' + ariaLabel + '"]';
		}

		return null;
	},

	getAriaLabelCssSelector : function(cur_element)
	{
		var ariaLabel = cur_element.getAttribute("aria-label");

		// If its not set return null
		if (ariaLabel == null || ariaLabel == '')
			return null;

		// Otherwise check if its unique for identifying the object in the document
		var elements = document.querySelectorAll(cur_element.tagName + '[aria-label=' + CSS.escape(ariaLabel) + ']');
		if (elements.length > 1) {
			// IF more than one found find the element
			for (var i = 0; i < elements.length; i++) {
				if (cur_element == elements[i]) {
					return cur_element.tagName + '[aria-label="' +  CSS.escape(ariaLabel) + '"]:nth-of-type(' + (i+1) + ')';
				}
			}
		} else if (elements.length == 1) {
			return cur_element.tagName + '[aria-label="' +  CSS.escape(ariaLabel) + '"]';
		}

		return null;
	},

	getDataIdXPath : function(cur_element)
	{
		var dataId = cur_element.getAttribute("data-id");

		// If its not set return null
		if (dataId == null || dataId == '')
			return null;

		var elements = document.querySelectorAll(cur_element.tagName + '[data-id=' + CSS.escape(dataId) + ']');
		if (elements.length > 1) {
			// IF more than one found find the element
			for (var i = 0; i < elements.length; i++) {
				if (cur_element == elements[i]) {
					return "(//" + cur_element.tagName + '[@data-id="' + dataId + '"])[' + (i+1) + ']';
				}
			}
		} else if (elements.length == 1) {
			return "//" + cur_element.tagName + '[@data-id="' + dataId + '"]';
		}

		return null;
	},

	getDataIdCssSelector : function(cur_element, shadowParent)
	{
		var dataId = cur_element.getAttribute("data-id");

		// If its not set return null
		if (dataId == null || dataId == '')
			return null;

		var elements;

		if (shadowParent != null && shadowParent.shadowRoot != null) {
			elements = shadowParent.shadowRoot.querySelectorAll(cur_element.tagName + '[data-id=' + CSS.escape(dataId) + ']');
		} else {
		 	elements = document.querySelectorAll('[data-id=' + CSS.escape(dataId) + ']');
	 	}

		if (elements.length > 1) {
			// IF more than one found find the element
			for (var i = 0; i < elements.length; i++) {
				if (cur_element == elements[i]) {
					return cur_element.tagName + '[data-id="' +  CSS.escape(dataId) + '"]:nth-of-type(' + (i+1) + ')';
				}
			}
		} else if (elements.length == 1) {
			return cur_element.tagName + '[data-id="' +  CSS.escape(dataId) + '"]';
		}

		return null;
	},

	getElementCssPath : function(cur_element)
	{
		return Xpath.getElementTreeCSS(cur_element);
	},

	getElementTreeXPath : function(cur_element)
	{
		return Xpath.getElementTreeXPath(cur_element);
	},

	getUniqueSelector : function(cur_element, strategy_list, actionType, scanOption)
	{
		if (scanOption == "ServiceNow") {
			if (actionType === Action_Type.EnterText) {
				var nameResult = globalSelector.getSelectorForStrategy(cur_element, strategy_list[SelectorStrategies.ServiceNowName]);
				if (nameResult != null)
					return nameResult;
			}

			var idResult = globalSelector.getSelectorForStrategy(cur_element, strategy_list[SelectorStrategies.ServiceNowId]);
			if (idResult != null)
				return idResult;

		} else {
			for (var i = 0; i < strategy_list.length; i++) {
				try {
					var result = globalSelector.getSelectorForStrategy(cur_element, strategy_list[i]);
				} catch (ex) {}

				if (result != null) {
					return result;
				}
			}
		}

		return null;
	},

	buildServiceNowSelectorObject : function(cur_element)
	{
		var builderObject = [];

		if (cur_element.tagName.toLowerCase() === "input") {
			var nameElem = globalSelector.getName(cur_element);

			if (nameElem != null)
				builderObject.push({identifierType: SelectorStrategies.ServiceNowName, identifierValue: nameElem, identifierTrust: 1.0});

		} else if (cur_element.tagName.toLowerCase() === "select") {
			var id = globalSelector.getObjectId(cur_element);
			if (id != null && id != '') {
				var idTrust = 1.0;

				builderObject.push({identifierType: SelectorStrategies.ServiceNowId, identifierValue: id + "::::" + cur_element.selectedIndex, identifierTrust: 1.0});
			}
		} else {
			var id = globalSelector.getObjectGsft_Id(cur_element);
			if (id != null && id != '') {
				var idTrust = 1.0;
				builderObject.push({identifierType: SelectorStrategies.ServiceNowId, identifierValue: id, identifierTrust: 1.0});
			}
		}

		return builderObject;
	},

	buildSelectorObject : function (cur_element)
	{
				var builderObject = [];

					// XPath
					var xPathElem = globalSelector.getXPath(cur_element);
					builderObject.push({identifierType: SelectorStrategies.X_Path, identifierValue: xPathElem, identifierTrust: 1.0});

					// class name
					var className = globalSelector.getObjectClass(cur_element);
					if (className != null && className != '') {
						var classTrust = 1.0 / globalSelector.getObjectsForClass(cur_element, className).length;

						builderObject.push({identifierType: SelectorStrategies.Class, identifierValue: className, identifierTrust: classTrust});
					}

					// Id
					var id = globalSelector.getObjectId(cur_element);
					if (id != null && id != '') {
						var idTrust = 1.0 / globalSelector.getObjectsForId(cur_element, id).length;

						builderObject.push({identifierType: SelectorStrategies.Id, identifierValue: id, identifierTrust: idTrust});
					}

					// Tag
					var tag = globalSelector.getObjectTag(cur_element);
					if (tag != null && tag != '') {
						var tagTrust = 1.0 / globalSelector.getObjectsForTag(cur_element, tag).length;

						builderObject.push({identifierType: SelectorStrategies.Tag, identifierValue: tag, identifierTrust: tagTrust});
					}

					// Value
					var value = globalSelector.getObjectValue(cur_element);
					if (value != null && value != '') {
						var valueTrust = 1.0 / globalSelector.getObjectsForValue(cur_element, value).length;

						builderObject.push({identifierType: SelectorStrategies.Value, identifierValue: value, identifierTrust: valueTrust});
					}

					// Text
					var textVal = globalSelector.getObjectText(cur_element);
					if (textVal != null && textVal != '') {
						var textNo = globalSelector.getObjectsForText(cur_element, textVal);
						var textTrust = 0;
						if (textNo != 0)
						 	var textTrust = 1.0 / textNo;

						builderObject.push({identifierType: SelectorStrategies.TextContent, identifierValue: textVal, identifierTrust: textTrust});
					}

				return builderObject;
	},

	getSelectorForStrategy : function(cur_element, strategy)
	{
		if (strategy == SelectorStrategies.Id) {
			return globalSelector.getUniqueId(cur_element);
		} else if (strategy == SelectorStrategies.Class) {
			return globalSelector.getUniqueClass(cur_element);
		} else if (strategy == SelectorStrategies.Tag) {
			return globalSelector.getUniqueTag(cur_element);
		} else if (strategy == SelectorStrategies.X_Path) {
			return globalSelector.getXPath(cur_element);
		} else if (strategy == SelectorStrategies.Value) {
			return globalSelector.getUniqueValue(cur_element);
		} else if (strategy == SelectorStrategies.Name) {
			return globalSelector.getUniqueName(cur_element);
		} else if (strategy == SelectorStrategies.ServiceNowName) {
			return globalSelector.getName(cur_element);
		} else if (strategy == SelectorStrategies.ServiceNowId) {
			return globalSelector.getObjectGsft_Id(cur_element);
		} else {
			return null;
		}
	}
};
