String.prototype.trim = function (length) {
  return this.length > length ? this.substring(0, length) : this;
}

var nodeAttribute = ["accept", "accept-charset","accesskey","action","align","allow","alt","async","autocapitalize","autocomplete","autofocus",
                    "autoplay","background","bgcolor","border","buffered","capture","challenge","charset","checked","cite","class","code","codebase","color","cols","colspan",
                    "content","contenteditable","contextmenu","controls","coords","crossorigin","csp", "datetime", "decoding", "default", "defer", "dir", "dirname", "disabled",
                    "download","draggable","enctype","enterkeyhint","for","form","formaction","formenctype","formmethod","formnovalidate","formtarget","headers","height","hidden",
                    "high","href","hreflang","http-equiv","icon","id","importance","integrity","intrinsicsize","inputmode","ismap","itemprop","keytype","kind","label","lang","language",
                    "loading","list","loop","low","manifest","max","maxlength","minlength","media","method","min","multiple","muted","name","novalidate","open","optimum","pattern","ping",
                    "placeholder","poster","preload","radiogroup","readonly","referrerpolicy","rel","required","reversed","rows","rowspan","sandbox","scope","scoped","selected","shape",
                    "size","sizes","slot","span","spellcheck","src","srcdoc","srclang","srcset","start","step","style","summary","tabindex","target","title","translate","type","usemap",
                    "value","width", "wrap", "data"];

var screenScraper = {
  identifierMaps: [],

  identifierPreferences: [],

  identifierAttrs: [],

  selectorType: null,

  customScan: null,

  searchDepth: 4,

  positionIndex: true,

  buildIdentifierFromDomRecursive : function (docElement, scan_option, shadowParent, expand)
  {

    return screenScraper.buildIdentifierFromDom(docElement, scan_option, shadowParent, expand);
  },

  buildIdentifierFromDom : function (docElement, scan_option, shadowParent, expand)
  {
      var object = null;

      if (docElement.tagName.toLowerCase() === "input" || docElement.tagName.toLowerCase() === "textarea") {
        try {
          if (docElement.getAttribute("type") != null && (docElement.getAttribute("type").toLowerCase() === "button" || docElement.getAttribute("type").toLowerCase() === "radio"  || docElement.getAttribute("type").toLowerCase() === "checkbox" || docElement.getAttribute("type").toLowerCase() === "submit")) {
            try {
              var parameters = [];
              if (customScan === "ServiceNow") {
                parameters = screenScraper.buildServiceNowSelectorObject(docElement);
              } else {
                parameters = screenScraper.buildSelectorObjectRecursive(docElement, shadowParent);
              }

              if (parameters.length > 0) {
                var elementLabel = englishBuilder.elementLabelName(docElement, globalSelector.getXPath(docElement));

                object = {
                  name: elementLabel.value.replace(/\s+/g, " ").trim(99),
                  nameWarning: elementLabel.warning,
                  objectType: "Button",
                  parameters: parameters
                };
              }
            } catch (err) {
              console.log(err);
            }
          } else {
            try {
              var parameters = [];

              if (customScan === "ServiceNow") {
                parameters = screenScraper.buildServiceNowSelectorObject(docElement);
              } else {
                parameters = screenScraper.buildSelectorObjectRecursive(docElement, shadowParent);
              }

              if (parameters.length > 0) {
                var dataType = "text";

                var elementLabel = englishBuilder.elementLabelName(docElement, globalSelector.getXPath(docElement));

                object = {
                  name: elementLabel.value.replace(/\s+/g, " ").trim(99),
                  nameWarning: elementLabel.warning,
                  objectType: "Input",
                  dataType: dataType,
                  parameters: parameters
                };
              }

            } catch (err) {
              console.log(err);
            }
          }
        } catch (err) {
          console.log(err);

          return null;
        }

      } else if (docElement.tagName.toLowerCase() === "option") {
        return screenScraper.buildIdentifierFromDom(docElement.parentNode, scan_option, shadowParent, expand);

      } else if (docElement.tagName.toLowerCase() === "select") {
        try {
          var parameters = [];

          if (customScan === "ServiceNow") {
            parameters = screenScraper.buildServiceNowSelectorObject(docElement);

          } else {
            parameters = screenScraper.buildSelectorObjectRecursive(docElement, shadowParent);//docElement.options[docElement.selectedIndex]);
          }

          if (parameters.length > 0) {
            var elementLabel = englishBuilder.elementLabelName(docElement, globalSelector.getXPath(docElement));

            object = {
              name: elementLabel.value.replace(/\s+/g, " ").trim(99).trim(),
              nameWarning: elementLabel.warning,
              objectType: "Select",
              dataType: "select",
              dataAttributes: JSON.stringify(screenScraper.getSelectValues(docElement)),
              parameters: parameters
            };
          }
        } catch (err) {
          console.log(err);

          return null;
       }

      } else if (docElement.tagName.toLowerCase() === "button" || screenScraper.isDescendant(docElement, 'button')) {
        try {
          var parameters = [];

          if (customScan === "ServiceNow") {
            parameters = screenScraper.buildServiceNowSelectorObject(docElement);
          } else {
            parameters = screenScraper.buildSelectorObjectRecursive(docElement, shadowParent);
          }
          if (parameters.length > 0) {
            var elementLabel = englishBuilder.elementLabelName(docElement, globalSelector.getXPath(docElement));

            object = {
              name: elementLabel.value.replace(/\s+/g, " ").trim(99),
              nameWarning: elementLabel.warning,
              objectType: "Button",
              parameters: parameters
            };
          }
        } catch (err) {
          console.log(err);

          return null;
        }

      } else if (docElement.tagName.toLowerCase() === "div") {
        if (expand) {
          var inputs = docElement.querySelectorAll('input');
          if (inputs.length >= 1) {
            return screenScraper.buildIdentifierFromDom(inputs[0], scan_option, shadowParent, expand);
          }


          var buttons = docElement.querySelectorAll('button');
          if (buttons.length >= 1) {
            return screenScraper.buildIdentifierFromDom(buttons[0], scan_option, shadowParent, expand);
          }
        }
      } else if (docElement.tagName.toLowerCase() === "a" || screenScraper.isDescendant(docElement, 'a')) {
        try {
          var parameters = [];

          if (customScan === "ServiceNow") {
            parameters = screenScraper.buildServiceNowSelectorObject(docElement);
          } else {
            parameters = screenScraper.buildSelectorObjectRecursive(docElement, shadowParent);
          }

          if (parameters.length > 0) {
            var elementLabel = englishBuilder.elementLabelName(docElement, globalSelector.getXPath(docElement));

            object = {
              name: elementLabel.value.replace(/\s+/g, " ").trim(99),
              nameWarning: elementLabel.warning,
              objectType: "Link",
              parameters: parameters
            };
          }
        } catch (err) {
          console.log(err);

          return null;
        }
      }

      if (object == null) {
        try {
          var parameters = [];

          if (customScan === "ServiceNow") {
            parameters = screenScraper.buildServiceNowSelectorObject(docElement);
          } else {
            parameters = screenScraper.buildSelectorObjectRecursive(docElement, shadowParent);
          }

          var objectType = "Link";
          if (parameters.length > 0) {
            var elementLabel = englishBuilder.elementLabelName(docElement, globalSelector.getXPath(docElement));

            object = {
              name: elementLabel.value.replace(/\s+/g, " ").trim(99),
              nameWarning: elementLabel.warning,
              objectType: objectType,
              parameters: parameters
            };
          }
        } catch (err) {
          console.log(err);

          return null;
        }
      }

      if (object != null) {
		  if (event != null && event.target != null) {
			object.coordinates = JSON.stringify({x: event.target.getBoundingClientRect().x + window.scrollX,
										 y: event.target.getBoundingClientRect().y + window.scrollY,
										 width: event.target.getBoundingClientRect().width,
										 height: event.target.getBoundingClientRect().height});
		  }
		  
        object.screenSize = JSON.stringify(ImageActionObject.ConstructEmptyObject());

        object.uniqueXPath = globalSelector.getElementTreeXPath(docElement);

        var iframeElem = screenScraper.buildIFrameXPath(docElement);
        object.iframeXPath = iframeElem.value;
        object.iframeWarning = iframeElem.warning;

        // Sort parameters according to priority they'll be consumed;
        object.parameters.sort(function (p1, p2) {
          if (p1.confidence > p2.confidence) return -1;
          if (p1.confidence < p2.confidence) return 1;

          if (p1.confidence == p2.confidence) {
            if (p1.attrPreference < p2.attrPreference) {
              return -1;
            } else if (p2.attrPreference < p1.attrPreference) {
              return 1;
            }

            if (p1.preferred && p2.preferred) {
              if (p1.identiferWarning) {
                return 1;
              } else if (p2.identiferWarning) {
                return -1;
              } else if ((!p1.shadowRequired) && p2.shadowRequired) {
                return -1;
              } else if ((p1.shadowRequired) && (!p2.shadowRequired)) {
                return 1;
              } else if (p1.paramValue.includes("data-id")) {
                return -1;
              } else if (p1.paramValue.includes("aria-label")) {
                return -1;
              } else if (p2.paramValue.includes("data-id")) {
                return 1;
              } else if (p2.paramValue.includes("aria-label")) {
                return 1;
              }
            } else if (p1.preferred) {
              return -1;
            } else if (p2.preferred) {
              return 1;
            } else {
              return 0;
            }
          } else {
            return 0;
          }
        });

        for (var i = 0; i < object.parameters.length; i++) {
          object.parameters[i].index = i;

          if (object.parameters[i].preferred != null)
            delete object.parameters[i].preferred;
        }
      }

      return object;
  },

  buildIFrameXPath : function (element, scan_option)
  {
    var id = [];

    var curWindow = window;
    while (curWindow != null && curWindow.frameElement != null) {
      id.unshift(screenScraper.buildShortXPath(curWindow.frameElement));

      curWindow = curWindow.parent;
    }

    if (id.length == 0) {
      if (window.self === window.top) {
        return {value: null, warning: false};
      } else {
        return {value: "Cross-site unknown", warning: true};
      }
    }

    var idStr = "";

    if (id.length > 1) {
      for (var i = 0; i < id.length; i++) {
        idStr = idStr + "[" + id[i] + "]";

        if (i != id.length - 1)
          idStr = idStr + ">";
      }
    } else {
      idStr = id[0];
    }

    return {value: idStr, warning: false};
  },

  screenScrape : function (scan_option)
  {
    var objects = [];

    // links
    var links = document.querySelectorAll('a');
    for (var item of links) {
        try {
          objects.push(screenScraper.buildIdentifierFromDom(item, scan_option, true));
        } catch (err) {}
    }


    // input
    var inputElements = document.querySelectorAll('input');
    for (var item of inputElements) {
        try {
          objects.push(screenScraper.buildIdentifierFromDom(item, scan_option, true));
        } catch (err) {}
    }

    // select
    var selectOptions = document.querySelectorAll('select');
    for (var item of selectOptions) {
        try {
          objects.push(screenScraper.buildIdentifierFromDom(item, scan_option, true));
        } catch (err) {}
    }

    // button
    var buttons = document.querySelectorAll('button');
    for (var item of buttons) {
        try {
          objects.push(screenScraper.buildIdentifierFromDom(item, scan_option, true));
        } catch (err) {}
    }

    var pCol = {name: document.title.trim(99),
                url: document.URL,
                pageObjects: objects};

    return pCol;
  },

  buildServiceNowSelectorObject: function (cur_element)
  {
    var builderObject = [];

    if (cur_element.tagName.toLowerCase() === "input") {
      var nameElem = globalSelector.getName(cur_element);

      if (nameElem != null)
        builderObject.push({paramType: 'ServiceNowName', paramValue: nameElem, confidence: 1.0});

    } else if (cur_element.tagName.toLowerCase() === "select") {
      var id = globalSelector.getObjectId(cur_element);
      if (id != null && id != '') {
        var idTrust = 1.0;

        builderObject.push({paramType: 'ServiceNowId', paramValue: id + "::::" + cur_element.selectedIndex, confidence: idTrust.toFixed(2)});
      }
    } else if (cur_element.tagName.toLowerCase() === "button"){
      var id = globalSelector.getObjectGsft_Id(cur_element);

      if (id != null && id != '') {
        var idTrust = 1.0;

        builderObject.push({paramType: 'ServiceNowId', paramValue: id, confidence: idTrust});
      }
    }

    return builderObject;
  },

  buildShortXPath : function (cur_element)
  {
      // While it has a parent
      var element = cur_element;
      var curChain = "";

      while (true) {
        // For current element pick an attribute -> construct xpath
        if (curChain !== "") {
          curChain = "/" + curChain;
        }

        if (element.getAttribute("id") != null) {
          curChain = element.tagName + "[@id='" + element.getAttribute("id") + "']" + curChain;

        } else if (element.getAttribute("name") != null) {
          curChain = element.tagName + "[@name='" + element.getAttribute("name") + "']" + curChain;

        } else if (element.getAttribute("value") != null) {
          curChain = element.tagName + "[@value='" + element.getAttribute("value") + "']" + curChain;

        } else if (element.getAttribute("formcontrolname") != null) {
          curChain = element.tagName + "[@formcontrolname='" + element.getAttribute("formcontrolname") + "']" + curChain;

        } else if (element.getAttribute("aria-label") != null) {
          curChain = element.tagName + "[@aria-label='" + element.getAttribute("aria-label") + "']" + curChain;

        } else if (element.getAttribute("data-id") != null) {
          curChain = element.tagName + "[@data-id='" + element.getAttribute("data-id") + "']" + curChain;

        } else if (element.getAttribute("class") != null) {
          curChain = element.tagName + "[@class='" + element.getAttribute("class") + "']" + curChain;

        } else {
          curChain = element.tagName + curChain;
        }

        // Check if xpath is unique
        try {
          var xPath = "//" + curChain;
          var headings = document.evaluate("count(" + xPath + ")", document, null, XPathResult.NUMBER_TYPE, null );

    			var noResults = headings.numberValue;

          if (noResults == 1) {
            return xPath;
          }

        } catch(err) {
          console.log(err);

    			return globalSelector.getElementTreeXPath(cur_element);
    		}

        // Go up to parent
        if (element.parentElement != null) {
          element = element.parentElement;
        } else {
          return globalSelector.getElementTreeXPath(cur_element);
        }
      }

      // Last resort
      return globalSelector.getElementTreeXPath(cur_element);
  },

  buildShortCssPath : function (cur_element, shadowParent)
  {
      // While it has a parent
      var element = cur_element;
      var curChain = "";

      while (true) {
        // For current element pick an attribute -> construct xpath
        if (curChain !== "") {
          curChain = ">" + curChain;
        }

        var tagName = (element.prefix ? element.prefix + ":" : "") + element.localName;

        if (element.getAttribute("id") != null) {
          curChain = tagName + "[id='" + CSS.escape(element.getAttribute("id")) + "']" + curChain;

        } else if (element.getAttribute("name") != null) {
          curChain = tagName + "[name='" + CSS.escape(element.getAttribute("name")) + "']" + curChain;

        } else if (element.getAttribute("value") != null) {
          curChain = tagName + "[value='" + CSS.escape(element.getAttribute("value")) + "']" + curChain;

        } else if (element.getAttribute("aria-label") != null) {
          curChain = tagName + "[aria-label='" + CSS.escape(element.getAttribute("aria-label")) + "']" + curChain;

        } else if (element.getAttribute("formcontrolname") != null) {
          curChain = tagName + "[formcontrolname='" + CSS.escape(element.getAttribute("formcontrolname")) + "']" + curChain;

        } else if (element.getAttribute("data-id") != null) {
          curChain = tagName + "[data-id='" + CSS.escape(element.getAttribute("data-id")) + "']" + curChain;

        } else if (element.getAttribute("class") != null) {
          curChain = tagName + "[class='" + CSS.escape(element.getAttribute("class")) + "']" + curChain;

        } else if (element.getAttribute("path") != null) {
          curChain = tagName + "[path='" + CSS.escape(element.getAttribute("path")) + "']" + curChain;

        } else {
          curChain = tagName + curChain;
        }

        // Check if xpath is unique
        try {
          var css = curChain;

          var elements;
          if (shadowParent != null && shadowParent.shadowRoot != null) {
      			elements = shadowParent.shadowRoot.querySelectorAll(css);
      		} else {
      		 	elements = document.querySelectorAll(css);
      	 	}

    			var noResults = elements.length;

          if (noResults == 1) {
            return css;
          }

        } catch(err) {
          console.log(err);

    			return globalSelector.getElementCssPath(cur_element);
    		}

        // Go up to parent
        if (element.parentElement != null) {
          element = element.parentElement;
        } else {
          return globalSelector.getElementCssPath(cur_element);
        }
      }

      // Last resort
      return globalSelector.getElementCssPath(cur_element);
  },

  buildShortCssPathShadowDom : function (cur_element, shadowParent)
  {
      // While it has a parent
      var element = cur_element;
      var curChain = "";

      while (true) {
        // For current element pick an attribute -> construct xpath
        if (curChain !== "") {
          curChain = ">" + curChain;
        }

        var tagName = (element.prefix ? CSS.escape(element.prefix) + ":" : "") + CSS.escape(element.localName);

        curChain = tagName + curChain;

        // Check if xpath is unique
        try {
          var css = curChain;

          var elements;
          if (shadowParent != null && shadowParent.shadowRoot != null) {
      			elements = shadowParent.shadowRoot.querySelectorAll(css);
      		} else {
      		 	elements = document.querySelectorAll(css);
      	 	}

    			var noResults = elements.length;

          if (noResults == 1) {
            return css;
          }

        } catch(err) {
          console.log(err);

    			return globalSelector.getElementCssPath(cur_element);
    		}

        // Go up to parent
        if (element.parentElement != null) {
          element = element.parentElement;
        } else {
          return globalSelector.getElementCssPath(cur_element);
        }
      }

      // Last resort
      return globalSelector.getElementCssPath(cur_element);
  },

  buildSelectorObjectRecursive : function (curElement, shadowParent)
  {
      // Build the identifiers
      var builderObject = [];

      for (var i = 0; i < screenScraper.identifierMaps.length; i++) {
        var map = screenScraper.identifierMaps[i].mapping;

        var curPathElem = screenScraper.buildShortXPath(curElement);
        if (curPathElem == null)
          continue;

        var mappedElement = curPathElem + "/" + map;

        var foundElements = globalSelector.getElementsByXPath(mappedElement,shadowParent);
        if (foundElements.length > 0) {
            var targetElement = foundElements[0];

            try {
              var identifiers = screenScraper.buildAncestorIdentifier(curElement, targetElement, shadowParent);
              if (identifiers != null)
                builderObject = builderObject.concat(identifiers);
            } catch (err) {
              console.log(err);
            }

        }
      }

      // Try and find a label
      if (curElement.id != null) {
        try {

          var labelElem;

          var labelXPath = '//label[@for="' + curElement.id +  '"]';
          if (screenScraper.customScan == "Sukgu" && shadowParent != null) {
            labelElem = getXPathObject(labelXPath);
          } else {
            labelElem = document.evaluate(labelXPath, document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
          }

          if (labelElem == null) {
            for (var i = 0; i < screenScraper.searchDepth; i++) {
              var curElem = curElement;
              for (var j = 0; j < i; j++) {
                if (curElem != null)
                  curElem = curElem.parentNode;
              }

              if (curElem != null && curElem.toString() !== "[object ShadowRoot]") {
                var elements = curElem.getElementsByTagName("label");

                if (elements.length == 1) {
                  labelElem = elements[0];
                }
              }

              if (labelElem != null)
                break;
            }
          }

          if (labelElem != null && labelElem.textContent != null) {
              var labelIdentifier = "//" + labelElem.tagName.toLowerCase() + "[normalize-space()= '" + labelElem.textContent.replace(/^\s+|\s+$/g, '') + "']";
              var foundElements = null;
              if (screenScraper.customScan == "Sukgu" && shadowParent != null) {
                foundElements = getXPathAllObjectUnique(labelIdentifier);
              } else {
                foundElements = globalSelector.getElementsByXPath(labelIdentifier,shadowParent);
              }

              // Build the index of the label if multiple are found ()
              if (foundElements != null && foundElements.length > 1) {
                var labelIndex = globalSelector.getElementIndex(foundElements);

                labelIdentifier = "(." + labelIdentifier + ")[" + (labelIndex + 1) + "]";
              }

              var identifiers = screenScraper.buildAncestorIdentifier(curElement, labelElem, shadowParent, labelIdentifier);

              if (identifiers != null)
                builderObject = builderObject.concat(identifiers);
          }
        } catch (err) {
          console.log(err);
        }
      }

      builderObject = builderObject.concat(screenScraper.buildSelectorObject(curElement, shadowParent));

      // Filter
      if (screenScraper.selectorType != null) {

        for (var i = builderObject.length - 1; i >= 0; i--) {
          if (screenScraper.selectorType === "CSS" && builderObject[i].paramType !== "CssSelector") {
            builderObject.splice(i, 1);
          } else if (screenScraper.selectorType === "XPath" && builderObject[i].paramType !== "XPath") {
            builderObject.splice(i, 1);
          }
        }
      }

      return builderObject;
  },

  buildAncestorIdentifier : function (curElement, targetElement, shadowParent, fixedId = null)
  {
    // Common ancestors
    function parents(node) {
      var nodes = []
      for (; node; node = node.parentNode) {
        nodes.unshift(node)
      }
      return nodes
    }

    function commonAncestor(node1, node2) {
      var parents1 = parents(node1)
      var parents2 = parents(node2)

      if (parents1[0] != parents2[0]) throw "No common ancestor!"

      for (var i = 0; i < parents1.length; i++) {
        if (parents1[i] != parents2[i]) return parents1[i - 1]
      }
    }

    var commonParent = commonAncestor(curElement, targetElement);
    if (commonParent == null)
      return null;

    // 1) Work our way from targetElement back to commonParent
    var listUp = [];
    var curElem = targetElement;

    var limit = 0;
    while (curElem != commonParent && limit < 100) {
      listUp.push(curElem.tagName);

      curElem = curElem.parentNode;

      limit++;
    }

    // 2) Work our way from commonParent to curElement
    var limit = 0;
    var listDown = [];
    while (curElem != curElement && limit < 100) {

      for (var i = 0; i < curElem.children.length; i++) {
        var childNode = curElem.children[i];

        if (childNode.contains(curElement)) {
          // How many have same tag name
          var index = 0;
          var noItems = 0;
          for (var j = 0; j < curElem.children.length; j++) {
            if (curElem.children[j] == childNode)
              break;

            if (curElem.children[j].tagName == childNode.tagName) {
              index++;
            }
          }

          for (var j = 0; j < curElem.children.length; j++) {
            if (curElem.children[j].tagName == childNode.tagName) {
              noItems++;
            }
          }

          curElem = childNode;

          listDown.push({tagName: curElem.tagName, elem: curElem, count: index, noItems: noItems });
          break;
        }
      }

      limit++;
    }

    // 3) Construct the related xpath
    var xPath = "";
    for (var i = 0; i < listUp.length; i++) {
      xPath = xPath + "/..";
    }

    for (var i = 0; i < listDown.length; i++) {
      var comp = listDown[i];

      if (comp.noItems > 1) {
        xPath = xPath  + "/" + comp.tagName.toLowerCase() + "[" + (comp.count + 1) + "]";
      } else {
        xPath = xPath + "/" + comp.tagName.toLowerCase();
      }
    }

    var identifiers = [];
    if (fixedId != null) {
      var builderObj = {paramType: 'XPath', paramValue: fixedId, preferred: true, shadowRequired: !(shadowParent == null || screenScraper.customScan == "Sukgu")};

      identifiers.push(builderObj);

    } else {
      identifiers = screenScraper.buildSelectorObject(targetElement, shadowParent);
    }

    for (var j = identifiers.length - 1; j >= 0; j--) {
      var ident = identifiers[j];

      if (ident.paramType === "XPath") {
        ident.paramValue = ident.paramValue + xPath;

        ident.confidence = screenScraper.getTrust(ident.paramValue, shadowParent).trust;

        ident.identiferWarning = screenScraper.checkIdentifier(ident.paramValue, ident.confidence);

      } else {
        identifiers.splice(j, 1);
      }
    }

    return identifiers;
  },

  getTrust : function (xPathElem, shadowParent)
  {
    var xPathelements = 0;
    if (screenScraper.customScan == "Sukgu" && shadowParent != null) {
      xPathelements = getXPathAllObjectUnique(xPathElem).length;
    } else {
      xPathelements = globalSelector.getNoElementsByXPath(xPathElem);
    }

    var trust = 1.0 / xPathelements;
    trust = isFinite(trust) && trust || 0; // updated

    return {trust: trust, mumber: xPathelements};
  },

  buildSelectorObject : function (cur_element, shadowParent)
	{
			var builderObject = [];

      var atts = cur_element.attributes;

      // Loop through all attributes
      // Scan all the attributes in the element - gives us a better chance of finding something
      // to use for the identifier
      for (var i = 0; i < atts.length; i++){
        var attrName = atts[i].nodeName;
        var attrValue = atts[i].nodeValue;

        if (attrValue == null || attrValue === "")
          continue;

        // XPath - Not supported for ShadowDom elements
        if (shadowParent == null || screenScraper.customScan == "Sukgu") {
          var xPathAttrName = attrName;
          if (nodeAttribute.indexOf(attrName) >= 0 || attrName.startsWith("data-")) {
            xPathAttrName = "@" + attrName;
          }

          try {
            var elementXPath = "//" + cur_element.tagName + "[" + xPathAttrName + "='" + screenScraper.escapeXpathString(attrValue) + "']"

            var matchingXPathElements = [];
            if (screenScraper.customScan == "Sukgu" && shadowParent != null) {
              matchingXPathElements = getXPathAllObjectUnique(elementXPath);
            } else {
              matchingXPathElements = globalSelector.getElementsByXPath(elementXPath);
            }

            noMatchingXPathElements = matchingXPathElements.length;
            if (screenScraper.positionIndex) {
              if (matchingXPathElements != null && matchingXPathElements.length > 1 && matchingXPathElements.length <= 4) {
                var elemIndex = globalSelector.getElementIndex(matchingXPathElements, cur_element);

                elementXPath = "(" + elementXPath + ")[" + (elemIndex + 1) + "]";

                noMatchingXPathElements = 1;
              }
            }

            var xpathIdentifierTrust = 1.0 / noMatchingXPathElements;
            xpathIdentifierTrust = isFinite(xpathIdentifierTrust) && xpathIdentifierTrust || 0;

            var builderObj = {paramType: 'XPath', paramValue: elementXPath, confidence: xpathIdentifierTrust.toFixed(2), preferred: screenScraper.isPreferedIdentifier(attrName), identiferWarning: screenScraper.checkIdentifier(elementXPath, xpathIdentifierTrust), shadowRequired: !(shadowParent == null || screenScraper.customScan == "Sukgu")};

            if (screenScraper.identifierPreferences.indexOf(attrName) >= 0) {
              builderObj.attrPreference = screenScraper.identifierPreferences.indexOf(attrName);
            } else {
              builderObj.attrPreference = screenScraper.identifierPreferences.length + 1;
            }

            builderObject.push(builderObj);
          } catch (err) {
            console.log("Error finding attribute '" + attrName + "' with value '" + attrValue + "'");
            console.log(err);
          }
        }

        // CSS
        try {
          var elementCSS =  cur_element.tagName + "[" + attrName + "='" + CSS.escape(attrValue) + "']";
          var noMatchingCSSElements = globalSelector.getNoElementsByCSS(elementCSS, shadowParent);
          var cssIdentifierTrust = 1.0 / noMatchingCSSElements;
          cssIdentifierTrust = isFinite(cssIdentifierTrust) && cssIdentifierTrust || 0;

          var builderObj = {paramType: 'CssSelector', paramValue: elementCSS, confidence: cssIdentifierTrust.toFixed(2), preferred: screenScraper.isPreferedIdentifier(attrName), identiferWarning: screenScraper.checkIdentifier(elementCSS, cssIdentifierTrust), shadowRequired: !(shadowParent == null)};

          if (screenScraper.identifierPreferences.indexOf(attrName) >= 0) {
            builderObj.attrPreference = screenScraper.identifierPreferences.indexOf(attrName);
          } else {
            builderObj.attrPreference = screenScraper.identifierPreferences.length + 1;
          }

          builderObject.push(builderObj);

        } catch (err) {
          console.log("Error finding attribute '" + attrName + "' with value '" + attrValue + "'");
          console.log(err);
        }
      }

      // XPath Tree
      if (shadowParent == null || screenScraper.customScan == "Sukgu") {
        // Short XPath
        try {
          var xPathElem = screenScraper.buildShortXPath(cur_element);
          if(xPathElem) {

            var noMatchingXPathElements = 0;
            if (screenScraper.customScan == "Sukgu" && shadowParent != null) {
              noMatchingXPathElements = getXPathAllObjectUnique(xPathElem).length;
            } else {
              noMatchingXPathElements = globalSelector.getNoElementsByXPath(xPathElem);
            }

            var xpathIdentifierTrust = 1.0 / noMatchingXPathElements;
            xpathIdentifierTrust = isFinite(xpathIdentifierTrust) && xpathIdentifierTrust || 0;

            builderObject.push({paramType: 'XPath', paramValue: xPathElem, confidence: xpathIdentifierTrust, identiferWarning: screenScraper.checkIdentifier(xPathElem, xpathIdentifierTrust), shadowRequired: !(shadowParent == null || screenScraper.customScan == "Sukgu")});
          }
        } catch (err) {
          console.log("Error finding element by xpath tree");
          console.log(err);
        }

        // Text
        if (cur_element.textContent != null && cur_element.textContent !== "") {
          try {
            var textXpath = "//" + cur_element.tagName + "[contains(text(),'" + cur_element.textContent.replace(/^\s+|\s+$/g, '') + "')]"

            var noMatchingXPathElements = 0;
            if (screenScraper.customScan == "Sukgu" && shadowParent != null) {
              noMatchingXPathElements = getXPathAllObjectUnique(textXpath).length;
            } else {
              noMatchingXPathElements = globalSelector.getNoElementsByXPath(textXpath);
            }

            if (noMatchingXPathElements != 0) {
              var textTrust = (1.0 / noMatchingXPathElements)
              textTrust = isFinite(textTrust) && textTrust || 0; // updated

              var identiferWarning = screenScraper.checkIdentifier(textXpath);

              var builderObj = {paramType: 'XPath', paramValue: textXpath, confidence: textTrust.toFixed(2), preferred: true, identiferWarning: identiferWarning, shadowRequired: !(shadowParent == null || screenScraper.customScan == "Sukgu")};
              if (screenScraper.identifierPreferences.indexOf("text") >= 0) {
                builderObj.attrPreference = screenScraper.identifierPreferences.indexOf("text");
              } else {
                builderObj.attrPreference = screenScraper.identifierPreferences.length + 1;
              }

              builderObject.push(builderObj);
            }
          } catch (err) {
            console.log("Error finding element by XPath text");

            console.log(err);
          }
        }
      }

      try {
        var xPathElem = screenScraper.buildShortCssPath(cur_element, shadowParent);
        if(xPathElem) {
          var identiferWarning = screenScraper.checkIdentifier(xPathElem);

          builderObject.push({paramType: 'CssSelector', paramValue: xPathElem, confidence: 1.0, identiferWarning: identiferWarning, shadowRequired: !(shadowParent == null)});
        }
      } catch (err) {
        console.log("Error finding element by xpath tree");
        console.log(err);
      }

      // text
      if (cur_element.textContent != null && cur_element.textContent !== "") {
        try {
          var textCSS =  cur_element.tagName + ":contains('" + CSS.escape(cur_element.textContent) + "')";

          var text_elements = globalSelector.getNoElementsByCSS(textCSS, shadowParent);
          if (text_elements != 0) {
            var textTrust = (1.0 / text_elements)
            textTrust = isFinite(textTrust) && textTrust || 0; // updated

            var identiferWarning = screenScraper.checkIdentifier(textCSS);

            var builderObj = {paramType: 'CssSelector', paramValue: textCSS, confidence: textTrust.toFixed(2), preferred: true, identiferWarning: identiferWarning, shadowRequired: !(shadowParent == null)};
            if (screenScraper.identifierPreferences.indexOf("text") >= 0) {
              builderObj.attrPreference = screenScraper.identifierPreferences.indexOf("text");
            } else {
              builderObj.attrPreference = screenScraper.identifierPreferences.length + 1;
            }

            builderObject.push(builderObj);
          }
        } catch (err) {
          console.log("Error finding element by text");
          console.log(err);
        }
      }

			return builderObject;
	},

  isPreferedIdentifier: function (attrName)
  {
    if (attrName.startsWith("data") || attrName.startsWith("aria") || attrName === "id" || attrName === "title" || attrName === "alt" || attrName === "form" || attrName === "label" || attrName === "name" || attrName === "placeholder" || attrName === "value")
      return true;

    return false;
  },

  checkIdentifier : function (elem, trust)
  {
    if (elem == null)
      return false;

    // Trust is low
    if (trust < 1.0)
      return true;

    // Look for identifiers which seem to contain random digits
    if (elem.replace(/[^0-9]/g,"").length > 5)
      return true;

    // Identifier seems long
    if (elem.length > 150 || elem.startsWith("/html") || elem.startsWith("html"))
      return true;

    return false;
  },

  getSelectValues : function(elem)
  {
    var values = [];

    if (elem.options == null)
      return values;

    for (var i = 0; i < elem.options.length; i++) {
      values.push(elem.options[i].text);
    }

    return values;
  },

  escapeXpathString: function (str)
  {
    return str;
  },

  isDescendant: function (child, type) {
       var node = child;
       while (node != null) {
           if (node.tagName != null && node.tagName.toLowerCase() === type) {
               return true;
           }
           node = node.parentNode;
       }
       return false;
  }
};
